#!/usr/bin/env python
#
# ___INFO__MARK_BEGIN__
#######################################################################################
# Copyright 2016-2024 Altair Engineering Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#######################################################################################
# ___INFO__MARK_END__
#

import os
import pwd
import socket
from uge import __version__

try:
    import UserDict
except ImportError:
    import collections as UserDict
try:
    import ConfigParser
except ImportError:
    import configparser as ConfigParser

# Defaults.
DEFAULT_UGE_ROOT = '/opt/uge'
DEFAULT_UGE_ETC = os.path.join(DEFAULT_UGE_ROOT, 'etc')
DEFAULT_UGE_BIN = os.path.join(DEFAULT_UGE_ROOT, 'bin')
DEFAULT_UGE_LOG_FILE = '/var/log/uge'
DEFAULT_UGE_CONFIG_FILE = os.path.join(
    DEFAULT_UGE_ETC, 'uge.conf')
DEFAULT_UGE_CONSOLE_LOG_LEVEL = 'info'
DEFAULT_UGE_FILE_LOG_LEVEL = 'debug'
DEFAULT_UGE_LOG_RECORD_FORMAT = \
    ('%(asctime)s,%(msecs)d [%(levelname)s] %(module)s:%(lineno)d'
     ' %(user)s@%(host)s %(name)s (%(process)d): %(message)s')
DEFAULT_UGE_LOG_DATE_FORMAT = '%Y-%m-%d %H:%M:%S'


class ConfigManager(UserDict.UserDict, object):
    """
    Singleton class used for keeping system configuration data. The class
    initializes its data using predefined defaults, or from the following
    environment variables:
        UGE_ROOT
        UGE_CONFIG_FILE
        UGE_LOG_FILE
        UGE_CONSOLE_LOG_LEVEL
        UGE_FILE_LOG_LEVEL

    Usage:
        from uge.config import config_manager
        cm = config_manager.ConfigManager()
        cm.set_console_log_level('info')
        level = cm.get_console_log_level()
        cm['my_key'] = 'my value'
        value = cm.get('my_key')
    """

    # Singleton.
    __instance = None

    def __new__(cls, *args, **kwargs):
        # Allow subclasses to create their own instances.
        if cls.__instance is None or cls != type(cls.__instance):
            instance = object.__new__(cls, *args, **kwargs)
            instance.__init__()
            cls.__instance = instance
        return cls.__instance

    @classmethod
    def get_instance(cls, *args, **kwargs):
        return cls.__new__(cls, *args, **kwargs)

    def __init__(self):
        """ Initialize configuration data. """
        # Only initialize once.
        if ConfigManager.__instance is not None:
            return
        UserDict.UserDict.__init__(self)
        self.config_parser = None
        self['defaultRoot'] = DEFAULT_UGE_ROOT
        self['defaultConfigFile'] = DEFAULT_UGE_CONFIG_FILE
        self['defaultLogFile'] = DEFAULT_UGE_LOG_FILE
        self['defaultConsoleLogLevel'] = DEFAULT_UGE_CONSOLE_LOG_LEVEL
        self['defaultFileLogLevel'] = DEFAULT_UGE_FILE_LOG_LEVEL
        self['defaultLogRecordFormat'] = DEFAULT_UGE_LOG_RECORD_FORMAT
        self['defaultLogDateFormat'] = DEFAULT_UGE_LOG_DATE_FORMAT

        # Settings that might come from environment variables.
        self.__set_from_env_variable('root', 'UGE_ROOT')
        self.__set_from_env_variable('logFile', 'UGE_LOG_FILE')
        self.__set_from_env_variable('configFile', 'UGE_CONFIG_FILE')
        self.__set_from_env_variable('consoleLogLevel', 'UGE_CONSOLE_LOG_LEVEL')
        self.__set_from_env_variable('fileLogLevel', 'UGE_FILE_LOG_LEVEL')

        # Variables affected by UGE_ROOT
        self['binDir'] = os.path.join(self.get_root(), 'bin')
        self['etcDir'] = os.path.join(self.get_root(), 'etc')

        # System info.
        self['host'] = socket.getfqdn()
        self['user'] = pwd.getpwuid(os.getuid())[0]

    # This function will ignore errors if environment variable is not set.
    def __set_from_env_variable(self, key, env_var):
        """
        Set value for the specified key from a given environment variable.
        This functions ignores errors for env. variables that are not set.
        """
        try:
            self[key] = os.environ[env_var]
        except: # pylint: disable=bare-except
            pass

    # This function will ignore errors if variable file is not present.
    def __set_from_var_file(self, key, var_file):
        """
        Set value for the specified key from a given file. The first line
        in the file is variable value.
        This functions ignores errors.
        """
        if os.path.exists(var_file):
            with open(var_file) as v:
                self[key] = v.readline().lstrip().rstrip()

    def __get_key_value(self, key, default='__internal__'):
        """
        Get value for a given key.
        Keys will be of the form 'logFile', and the default keys have
        the form 'defaultLogFile'.
        """
        default_key = "default" + key[0].upper() + key[1:]
        default_value = self.get(default_key, None)
        if default != '__internal__':
            default_value = default
        return self.get(key, default_value)

    def clear_config_parser(self):
        """ Clear config parser. """
        self.config_parser = None

    def get_config_parser(self, defaults=None):
        """ Return config parser, or none if config file cannot be found. """
        if defaults is None:
            defaults = {}
        if self.config_parser is None:
            config_file = self.get_config_file()
            self.config_parser = ConfigParser.ConfigParser(defaults)
            if os.path.exists(config_file):
                self.config_parser.read(config_file)
        if self.config_parser is not None:
            self.config_parser.defaults = defaults
        return self.config_parser

    def get_host(self):
        """ Get machine hostname. """
        return self['host']

    def get_user(self):
        """ Get user. """
        return self['user']

    def set_root(self, root):
        """ Set root. """
        self['root'] = root

    def get_root(self, default='__internal__'):
        """
        Get root. If the root has not been set, the function
        will return the specified default value. If the default value is
        not specified, internal (predefined) default will be returned.
        """
        return self.__get_key_value('root', default)

    def get_bin_dir(self, default='__internal__'):
        return self.__get_key_value('binDir', default)

    def get_etc_dir(self, default='__internal__'):
        return self.__get_key_value('etcDir', default)

    def set_config_file(self, config_file):
        """ Set config file. """
        self['configFile'] = config_file
        self.config_parser = None

    def get_config_file(self, default='__internal__'):
        """
        Get configuration file. If the configuration file has not
        been set, the function will return the specified default value.
        If the default value is not specified, internal (predefined)
        default will be returned.
        """
        return self.__get_key_value('configFile', default)

    def set_log_file(self, log_file):
        """ Set log file. """
        self['logFile'] = log_file

    def get_log_file(self, default='__internal__'):
        """
        Get log file. If the log file has not been set, the function
        will return the specified default value. If the default value is
        not specified, internal (predefined) default will be returned.
        """
        return self.__get_key_value('logFile', default)

    def set_console_log_level(self, level):
        """ Set user log level. """
        self['consoleLogLevel'] = level

    def get_console_log_level(self, default='__internal__'):
        """
        Get user log level. If the user log level has not
        been set, the function will return the specified default value.
        If the default value is not specified, internal (predefined)
        default will be returned.
        """
        return self.__get_key_value('consoleLogLevel', default)

    def set_file_log_level(self, level):
        """ Set system log level. """
        self['fileLogLevel'] = level

    def get_file_log_level(self, default='__internal__'):
        """
        Get system log level. If the system log level has not
        been set, the function will return the specified default value.
        If the default value is not specified, internal (predefined)
        default will be returned.
        """
        return self.__get_key_value('fileLogLevel', default)

    def set_log_record_format(self, format_):
        """ Set log record format. """
        self['logRecordFormat'] = format_

    def get_log_record_format(self, default='__internal__'):
        """
        Get log record format. If the log record format has not
        been set, the function will return the specified default value.
        If the default value is not specified, internal (predefined)
        default will be returned.
        """
        return self.__get_key_value('logRecordFormat', default)

    def get_log_date_format(self, default='__internal__'):
        """
        Get log date (timestamp) format. If the log date format has not
        been set, the function will return the specified default value.
        If the default value is not specified, internal (predefined)
        default will be returned.
        """
        return self.__get_key_value('logDateFormat', default)

    def set_config_defaults(self, defaults=None):
        """ Set configuration defaults. """
        if defaults is None:
            defaults = {}
        config_parser = self.get_config_parser()
        if config_parser is not None:
            config_parser.defaults = defaults

    def get_config_option(self, config_section, key, default_value=None):
        """ Get specified option from the configuration file. """
        config_parser = self.get_config_parser()
        if self.has_config_section(config_section):
            try:
                return config_parser.get(config_section, key, raw=True)
            except ConfigParser.NoOptionError:
                # ok, return default
                pass
        return default_value

    def get_config_sections(self):
        """ Return a list of the sections from the config file """
        config_parser = self.get_config_parser()
        if config_parser is not None:
            return config_parser.sections()
        return []

    def has_config_section(self, name):
        """ Return true if parser has config section """
        config_sections = self.get_config_sections()
        if name in config_sections:
            return True
        return False

    def get_config_items(self, config_section):
        """ Get available (key,value) pairs from the configuration file. """
        config_parser = self.get_config_parser()
        if config_parser is not None and \
                config_parser.has_section(config_section):
            return config_parser.items(config_section)
        return []

    def get_version(self):
        """ Get software version."""
        return __version__


#############################################################################
# Testing
if __name__ == '__main__':
    cm1 = ConfigManager.get_instance()
    print('CONFIG_FILE: ', cm1.get_config_file())
    cm1.set_config_file('/tmp/xyz')
    print('CONFIG_FILE: ', cm1.get_config_file())
    cm2 = ConfigManager()
    print('cm1 = cm2:', cm1 == cm2)
    print('ROOT: ', cm1.get_root())
    print('CONFIG_FILE: ', cm1.get_config_file())
    print('CONFIG FILE SECTIONS: ', cm1.get_config_sections())



#!/usr/bin/env python
#
# ___INFO__MARK_BEGIN__
#######################################################################################
# Copyright 2016-2024 Altair Engineering Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#######################################################################################
# ___INFO__MARK_END__
#
from .qconf_object import QconfObject


class User(QconfObject):
    """ This class encapsulates UGE user object. """

    #: Object version.
    VERSION = '1.0'

    #: Object name key.
    NAME_KEY = 'name'

    #: Object keys that must be provided by user.
    USER_PROVIDED_KEYS = ['name']

    #: Default values for required data keys.
    REQUIRED_DATA_DEFAULTS = {
        'oticket': 0,
        'fshare': 0,
        'delete_time': 0,
        'default_project': None,
    }

    INT_KEY_MAP = QconfObject.get_int_key_map(REQUIRED_DATA_DEFAULTS)
    FLOAT_KEY_MAP = QconfObject.get_float_key_map(REQUIRED_DATA_DEFAULTS)

    def __init__(self, name=None, data=None, metadata=None, json_string=None):
        """ 
        Class constructor. 

        :param name: User name. If provided, it will override user name from data or JSON string parameters ('name' key).
        :type name: str

        :param data: User data. If provided, it will override corresponding data from user JSON string representation.
        :type data: dict

        :param metadata: User metadata. If provided, it will override corresponding metadata from user JSON string representation.
        :type metadata: dict

        :param json_string: User JSON string representation.
        :type json_string: str

        :raises: **InvalidArgument** - in case metadata is not a dictionary, JSON string is not valid, or it does not contain dictionary representing an User object.

        """
        QconfObject.__init__(self, name=name, data=data, metadata=metadata, json_string=json_string)



#!/usr/bin/env python
#
# ___INFO__MARK_BEGIN__
#######################################################################################
# Copyright 2016-2024 Altair Engineering Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#######################################################################################
# ___INFO__MARK_END__
#
from uge.exceptions.invalid_argument import InvalidArgument
from .qconf_object import QconfObject


class ComplexConfigurationBase(QconfObject):
    """ This class serves as a base for UGE complex configuration objects. """

    DEFAULT_DICT_DELIMITER = ' '
    DICT_KEY_MAP = {
    }

    UGE_PYTHON_OBJECT_MAP = {
        'NONE': None,
        'YES': True,
        'NO': False,
    }

    UGE_PYTHON_BOOL_VALUE_MAP = {
        'TRUE': True,
        'FALSE': False,
    }

    UGE_PYTHON_TYPE_MAP = {
        'BOOL': int,
        'INT': int,
        'FLOAT': float,
        'DOUBLE': float,
    }

    OPTIONAL_KEYS_ALLOWED = True

    def __init__(self, data=None, metadata=None, json_string=None):
        """ 
        Class constructor. 

        :param data: Configuration data. If provided, it will override corresponding data from JSON string representation.
        :type data: dict

        :param metadata: Configuration metadata. If provided, it will override corresponding metadata from JSON string representation.
        :type metadata: dict

        :param json_string: Configuration JSON string representation.
        :type json_string: str

        :raises: **InvalidArgument** - in case metadata is not a dictionary, JSON string is not valid, or it does not contain dictionary representing a ClusterConfiguration object.
        """

        QconfObject.__init__(self, data=data, metadata=metadata, json_string=json_string)

    def to_uge(self):
        """ 
        Converts object to string acceptable as input for UGE qconf command.

        :returns: Object's UGE-formatted string.
        """
        lines = ''
        lines += '#name               shortcut   type        relop requestable consumable default  urgency aapre affinity do_report is_static\n'
        lines += '#------------------------------------------------------------------------------------------------------\n'
        for (key, value_dict) in list(self.data.items()):
            lines += '%s' % (key)
            for key2 in ['shortcut', 'type', 'relop', 'requestable', 'consumable', 'default', 'urgency', 'aapre',
                         'affinity', 'do_report', 'is_static']:
                lines += ' %s' % (self.py_to_uge(key2, value_dict[key2], value_dict.get('default_is_bool', False)))
            lines += '\n'
        return lines

    def convert_data_to_uge_keywords(self, data):
        for (_, value_dict) in list(data.items()):
            for (key, value) in list(value_dict.items()):
                value_dict[key] = self.py_to_uge(key, value, value_dict.get('default_is_bool', False))

    def py_to_uge(self, key, value, default_is_bool=False):
        items = list(self.UGE_PYTHON_OBJECT_MAP.items())
        if key == 'default' and default_is_bool:
            items = list(self.UGE_PYTHON_BOOL_VALUE_MAP.items())
        for (uge_value, py_value) in items:
            if value == py_value and type(value) == type(py_value):
                return uge_value
        return value

    def uge_to_py(self, key, value, uge_type=None):
        uppercase_value = value.upper()
        for (uge_value, py_value) in list(self.UGE_PYTHON_OBJECT_MAP.items()):
            if uge_value == uppercase_value:
                return py_value
        for (uge_value, py_value) in list(self.UGE_PYTHON_BOOL_VALUE_MAP.items()):
            if uge_value == uppercase_value:
                return py_value
        if uge_type and uge_type in self.UGE_PYTHON_TYPE_MAP:
            py_value = self.UGE_PYTHON_TYPE_MAP[uge_type](value)
            return py_value
        return value

    def is_uge_value_bool(self, value):
        uppercase_value = value.upper()
        for (uge_value, _) in list(self.UGE_PYTHON_BOOL_VALUE_MAP.items()):
            if uge_value == uppercase_value:
                return True
        return False

    def to_dict(self, input_string):
        lines = input_string.split('\n')
        object_data = {}
        for line in lines:
            if not line:
                continue
            if line.startswith('#'):
                continue
            key_value = line.split()
            key = key_value[0]
            shortcut = key_value[1]
            uge_type = key_value[2]
            relop = key_value[3]
            requestable = self.uge_to_py(key, key_value[4])
            consumable = self.uge_to_py(key, key_value[5])
            default = self.uge_to_py(key, key_value[6], uge_type)
            default_is_bool = self.is_uge_value_bool(key_value[6])
            urgency = self.uge_to_py(key, key_value[7], 'INT')
            aapre = self.uge_to_py(key, key_value[8])
            affinity = self.uge_to_py(key, key_value[9])
            do_report = self.uge_to_py(key, key_value[10])
            is_static = self.uge_to_py(key, key_value[11])
            object_data[key] = {
                'shortcut': shortcut,
                'type': uge_type,
                'relop': relop,
                'requestable': requestable,
                'consumable': consumable,
                'default': default,
                'urgency': urgency,
                'aapre': aapre,
                'affinity': affinity,
                'do_report': do_report,
                'is_static': is_static,
                'default_is_bool': default_is_bool
            }
        return object_data

    def check_attribute_data(self, name, data):
        if not name:
            raise InvalidArgument('Invalid complex attribute name.')
        if type(data) != dict:
            raise InvalidArgument('Complex attribute data must be a dictionary.')
        for key in ['shortcut', 'type', 'relop', 'requestable', 'consumable', 'default', 'urgency', 'aapre',
                    'affinity', 'do_report', 'is_static']:
            if key not in data:
                raise InvalidArgument('Complex attribute data is missing the "%s" key.' % key)



#!/usr/bin/env python
#
# ___INFO__MARK_BEGIN__
#######################################################################################
# Copyright 2016-2024 Altair Engineering Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#######################################################################################
# ___INFO__MARK_END__
#

import tempfile
import types

from .utils import needs_uge
from .utils import generate_random_string
from .utils import create_config_file
from .utils import load_values

from uge.api.qconf_api import QconfApi
from uge.config.config_manager import ConfigManager
from uge.log.log_manager import LogManager
from uge.exceptions.object_not_found import ObjectNotFound
from uge.exceptions.object_already_exists import ObjectAlreadyExists

create_config_file()
API = QconfApi()
QUEUE_NAME = '%s.q' % generate_random_string(6)
CONFIG_MANAGER = ConfigManager.get_instance()
LOG_MANAGER = LogManager.get_instance()
VALUES_DICT = load_values('test_values.json')
print(VALUES_DICT)


@needs_uge
def test_object_not_found():
    try:
        q = API.get_queue('__non_existent__.q')
        assert (False)
    except ObjectNotFound as ex:
        # ok
        pass


def test_generate_queue():
    q = API.generate_queue(QUEUE_NAME)
    assert (q.data['qname'] == QUEUE_NAME)


def test_list_queues():
    ql = API.list_queues()
    assert (ql is not None)


def test_add_queue():
    ql = API.list_queues()
    q = API.add_queue(name=QUEUE_NAME)
    assert (q.data['qname'] == QUEUE_NAME)
    ql2 = API.list_queues()
    assert (len(ql2) == len(ql) + 1)
    assert (ql2.count(QUEUE_NAME) == 1)


def test_object_already_exists():
    try:
        q = API.add_queue(name=QUEUE_NAME)
        assert (False)
    except ObjectAlreadyExists as ex:
        # ok
        pass


def test_get_queue():
    q = API.get_queue(QUEUE_NAME)
    assert (q.data['qname'] == QUEUE_NAME)


def test_generate_queue_from_json():
    queue = API.get_queue(QUEUE_NAME)
    json = queue.to_json()
    queue2 = API.generate_object(json)
    assert (queue2.__class__.__name__ == queue.__class__.__name__)
    for key in list(queue.data.keys()):
        v = queue.data[key]
        v2 = queue2.data[key]
        if type(v) == list:
            assert (len(v) == len(v2))
            for s in v:
                assert (v2.count(s) == 1)
        elif type(v) == dict:
            for key in list(v.keys()):
                assert (str(v[key]) == str(v2[key]))
        else:
            assert (str(v) == str(v2))


def test_modify_queue():
    q = API.get_queue(QUEUE_NAME)
    slots = int(q.data['slots'][0])
    q = API.modify_queue(name=QUEUE_NAME, data={'slots': [str(slots + 1)]})
    slots2 = int(q.data['slots'][0])
    assert (slots2 == slots + 1)


def test_get_queue():
    queuel = API.list_queues()
    queues = API.get_queues()
    for queue in queues:
        print("#############################################")
        print(queue.to_uge())
        assert (queue.data['qname'] in queuel)


def test_write_queues():
    try:
        tdir = tempfile.mkdtemp()
        print("*************************** " + tdir)
        queue_names = VALUES_DICT['queue_names']
        queues = API.get_queues()
        for queue in queues:
            print("Before #############################################")
            print(queue.to_uge())

        new_queues = []
        for name in queue_names:
            nqueue = API.generate_queue(name=name)
            new_queues.append(nqueue)
        API.mk_queues_dir(tdir)
        API.write_queues(new_queues, tdir)
        API.add_queues_from_dir(tdir)
        API.modify_queues_from_dir(tdir)
        queues = API.get_queues()
        for queue in queues:
            print("After #############################################")
            print(queue.to_uge())

        queues = API.list_queues()
        for name in queue_names:
            assert (name in queues)
            print("queue found: " + name)

    finally:
        API.delete_queues_from_dir(tdir)
        API.rm_queues_dir(tdir)


def test_add_queues():
    try:
        new_queues = []
        queue_names = VALUES_DICT['queue_names']
        for name in queue_names:
            nqueue = API.generate_queue(name=name)
            new_queues.append(nqueue)

        # print all queues currently in the cluster
        queues = API.get_queues()
        for queue in queues:
            print("Before #############################################")
            print(queue.to_uge())

        # add queues
        API.add_queues(new_queues)
        API.modify_queues(new_queues)

        # print all queues currently in the cluster
        queues = API.get_queues()
        for queue in queues:
            print("After #############################################")
            print(queue.to_uge())

        # check that queues have been added
        queues = API.list_queues()
        for name in queue_names:
            assert (name in queues)
            print("queue found: " + name)

    finally:
        API.delete_queues(new_queues)


def test_delete_queue():
    ql = API.list_queues()
    API.delete_queue(QUEUE_NAME)
    ql2 = API.list_queues()
    assert (len(ql2) == len(ql) - 1)
    assert (ql2.count(QUEUE_NAME) == 0)



#!/usr/bin/env python
# 
# ___INFO__MARK_BEGIN__
#######################################################################################
# Copyright 2016-2024 Altair Engineering Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#######################################################################################
# ___INFO__MARK_END__
# 
import types
from .utils import needs_uge
from .utils import generate_random_string
from .utils import create_config_file

from uge.api.qconf_api import QconfApi

create_config_file()
API = QconfApi()


@needs_uge
def test_generate_conf():
    conf = API.generate_conf(name='global')
    assert ('max_u_jobs' in conf.data)


def test_get_conf():
    conf = API.get_conf()
    assert ('max_u_jobs' in conf.data)


def test_generate_conf_from_json():
    conf = API.get_conf(name='global')
    json = conf.to_json()
    conf2 = API.generate_object(json)
    assert (conf2.__class__.__name__ == conf.__class__.__name__)
    for key in list(conf.data.keys()):
        v = conf.data[key]
        v2 = conf2.data[key]
        if type(v) == list:
            assert (len(v) == len(v2))
            for s in v:
                assert (v2.count(s) == 1)
        elif type(v) == dict:
            for key in list(v.keys()):
                assert (str(v[key]) == str(v2[key]))
        else:
            assert (str(v) == str(v2))


def test_modify_conf():
    conf = API.get_conf()
    max_u_jobs = conf.data['max_u_jobs']
    conf = API.modify_conf(name='global', data={'max_u_jobs': max_u_jobs * 2})
    max_u_jobs2 = conf.data['max_u_jobs']
    assert (max_u_jobs2 == max_u_jobs * 2)
    conf = API.modify_conf(data={'max_u_jobs': max_u_jobs})
    max_u_jobs3 = conf.data['max_u_jobs']
    assert (max_u_jobs3 == max_u_jobs)



#!/usr/bin/env python
# 
# ___INFO__MARK_BEGIN__
#######################################################################################
# Copyright 2016-2024 Altair Engineering Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#######################################################################################
# ___INFO__MARK_END__
# 
import string
import random
import os
import os.path
import json
from tempfile import NamedTemporaryFile

from nose import SkipTest
from nose.tools import make_decorator

CONFIG_FILE = '/tmp/uge.test.conf'
LOG_FILE = '/tmp/uge.test.log'

CONFIG_FILE_TEMPLATE = """
[LoggerLevels]
root=error
expressions: ^.*$=trace

[ConsoleLogging]
handler=stream_log_handler.StreamLogHandler(sys.stdout,)
level=debug
format=%(asctime)s %(levelname)s %(process)d %(filename)s:%(lineno)d %(message)s
datefmt=%Y-%m-%d %H:%M:%S

[FileLogging]
handler=timed_rotating_file_log_handler.TimedRotatingFileLogHandler('LOG_FILE')
level=trace
format=%(asctime)s %(levelname)s %(process)d %(message)s
datefmt=%Y-%m-%d %H:%M:%S
"""


def create_config_file(use_temporary_file=False):
    config_file_name = CONFIG_FILE
    if use_temporary_file:
        config_file = NamedTemporaryFile(delete=False)
        config_file_name = config_file.name
    else:
        if os.path.exists(config_file_name):
            create_config_manager()
            return
        config_file = open(config_file_name, 'w')
    config_string = CONFIG_FILE_TEMPLATE.replace(
        'LOG_FILE', LOG_FILE)
    config_file.write(config_string)
    config_file.close()
    create_config_manager()
    return config_file_name


def remove_file(file_path):
    if os.path.exists(file_path):
        os.remove(file_path)


def remove_test_log_file():
    remove_file(LOG_FILE)


def remove_test_config_file():
    remove_file(CONFIG_FILE)


def remove_test_files():
    remove_file(LOG_FILE)
    remove_file(CONFIG_FILE)


def read_last_line(file_path):
    f = open(file_path, 'r')
    last_line = None
    while True:
        line = f.readline()
        if not line:
            break
        last_line = line
    f.close()
    return last_line


def read_last_log_line():
    return read_last_line(LOG_FILE)


def create_config_manager():
    from uge.config.config_manager import ConfigManager
    cm = ConfigManager.get_instance()
    cm.set_config_file(CONFIG_FILE)
    cm.set_log_file(LOG_FILE)
    cm.set_file_log_level('trace')


def generate_random_string(size, chars=string.ascii_lowercase + string.ascii_uppercase + string.digits):
    return ''.join(random.choice(chars) for _ in range(size))


def generate_random_string_list(n_strings, string_length, delimiter=',', string_prefix=''):
    string_list = ''
    string_delimiter = ''
    for i in range(0, n_strings):
        string_list = '%s%s%s%s' % (string_list, string_delimiter,
                                    string_prefix,
                                    generate_random_string(string_length))
        string_delimiter = delimiter
    return string_list


def load_values(value_file):
    tpd = {}
    if os.path.exists(value_file):
        tpd = json.load(open(value_file))
    return tpd


# Common decorators
def needs_setup(func):
    def inner(*args, **kwargs):
        create_config_file()
        return func(*args, **kwargs)

    return make_decorator(func)(inner)


def needs_cleanup(func):
    def inner(*args, **kwargs):
        remove_test_files()
        return func(*args, **kwargs)

    return make_decorator(func)(inner)


def needs_config(func):
    def inner(*args, **kwargs):
        try:
            create_config_manager()
        except Exception as ex:
            print(ex)
            raise SkipTest("Config manager instance could not be created.")
        return func(*args, **kwargs)

    return make_decorator(func)(inner)


def needs_uge(func):
    def inner(*args, **kwargs):
        from uge.exceptions.configuration_error import ConfigurationError
        if 'SGE_ROOT' not in os.environ:
            raise ConfigurationError('SGE_ROOT is not defined.')
        return func(*args, **kwargs)

    return make_decorator(func)(inner)


#############################################################################
# Testing
if __name__ == '__main__':
    # print 'Last line: ', read_last_line('/tmp/uge.log')
    create_config_file()
    d = load_values('test_values.json')
    print(d)



#!/usr/bin/env python
#
# ___INFO__MARK_BEGIN__
#######################################################################################
# Copyright 2016-2024 Altair Engineering Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#######################################################################################
# ___INFO__MARK_END__
#
from .qconf_object import QconfObject


class JobClass(QconfObject):
    """ This class encapsulates UGE job class object. """

    #: Object version.
    VERSION = '4.0'

    #: Object name key.
    NAME_KEY = 'jcname'

    #: Object keys that must be provided by user.
    USER_PROVIDED_KEYS = ['jcname']

    #: Default values for required data keys.
    REQUIRED_DATA_DEFAULTS = {
        'variant_list': None,
        'owner': None,
        'user_lists': None,
        'xuser_lists': None,
        'A': '{+}UNSPECIFIED',
        'a': '{+}UNSPECIFIED',
        'ar': '{+}UNSPECIFIED',
        'b': '{+}UNSPECIFIED',
        'binding': '{+}UNSPECIFIED',
        'c_interval': '{+}UNSPECIFIED',
        'c_occasion': '{+}UNSPECIFIED',
        'CMDNAME': '{+}UNSPECIFIED',
        'CMDARG': '{+}UNSPECIFIED',
        'ckpt': '{+}UNSPECIFIED',
        'ac': '{+}UNSPECIFIED',
        'cwd': '{+}UNSPECIFIED',
        'dl': '{+}UNSPECIFIED',
        'e': '{+}UNSPECIFIED',
        'h': '{+}UNSPECIFIED',
        'hold_jid': '{+}UNSPECIFIED',
        'hold_jid_ad': '{+}UNSPECIFIED',
        'i': '{+}UNSPECIFIED',
        'j': '{+}UNSPECIFIED',
        'js': '{+}UNSPECIFIED',
        'm': '{+}UNSPECIFIED',
        'mbind': '{+}UNSPECIFIED',
        'M': '{+}UNSPECIFIED',
        'N': '{+}UNSPECIFIED',
        'notify': '{+}UNSPECIFIED',
        'now': '{+}UNSPECIFIED',
        'o': '{+}UNSPECIFIED',
        'P': '{+}UNSPECIFIED',
        'p': '{+}UNSPECIFIED',
        'pe_name': '{+}UNSPECIFIED',
        'pe_range': '{+}UNSPECIFIED',
        'R': '{+}UNSPECIFIED',
        'r': '{+}UNSPECIFIED',
        'rou': '{+}UNSPECIFIED',
        'S': '{+}UNSPECIFIED',
        'shell': '{+}UNSPECIFIED',
        't': '{+}UNSPECIFIED',
        'tc': '{+}UNSPECIFIED',
        'V': '{+}UNSPECIFIED',
        'v': '{+}UNSPECIFIED',
        'xd': '{+}UNSPECIFIED',
        'rdi': '{+}UNSPECIFIED',
        'umask': '{+}UNSPECIFIED',
        'l_hard': '{+}UNSPECIFIED',
        'l_soft': '{+}UNSPECIFIED',
        'q_hard': '{+}UNSPECIFIED',
        'q_soft': '{+}UNSPECIFIED',
        'par': '{+}UNSPECIFIED',
    }

    INT_KEY_MAP = QconfObject.get_int_key_map(REQUIRED_DATA_DEFAULTS)
    FLOAT_KEY_MAP = QconfObject.get_float_key_map(REQUIRED_DATA_DEFAULTS)
    DEFAULT_LIST_DELIMITER = ','
    LIST_KEY_MAP = {
        'variant_list': ',',
        'user_lists': ',',
        'xuser_lists': ',',
    }

    def __init__(self, name=None, data=None, metadata=None, json_string=None):
        """ 
        Class constructor. 

        :param name: Job class name. If provided, it will override job class name from data or JSON string parameters ('jcname' key).
        :type name: str

        :param data: Job class data. If provided, it will override corresponding data from job class JSON string representation.
        :type data: dict

        :param metadata: Job class metadata. If provided, it will override corresponding metadata from job class JSON string representation.
        :type metadata: dict

        :param json_string: Job class JSON string representation.
        :type json_string: str

        :raises: **InvalidArgument** - in case metadata is not a dictionary, JSON string is not valid, or it does not contain dictionary representing a JobClass object.
        """

        QconfObject.__init__(self, name=name, data=data, metadata=metadata, json_string=json_string)



#!/usr/bin/env python
#
# ___INFO__MARK_BEGIN__
#######################################################################################
# Copyright 2016-2024 Altair Engineering Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#######################################################################################
# ___INFO__MARK_END__
#
import re
from uge.exceptions.invalid_request import InvalidRequest
from uge.exceptions.invalid_argument import InvalidArgument
from uge.exceptions.object_already_exists import ObjectAlreadyExists
from uge.exceptions.object_not_found import ObjectNotFound
from uge.objects.qconf_object_factory import QconfObjectFactory
from .dict_based_object_manager import DictBasedObjectManager


class ComplexConfigurationManager(DictBasedObjectManager):
    QCONF_ERROR_REGEX_LIST = [
        (re.compile('.*is not a valid.*'), InvalidRequest),
        (re.compile('.*cannot be deleted.*'), InvalidRequest),
        (re.compile('.*error parsing value.*'), InvalidArgument),
        (re.compile('.*should end.*'), InvalidArgument),
    ]

    # Failure incorrectly classified as successful outcome
    QCONF_FAILURE_REGEX_LIST = []

    GENERATE_OBJECT_FACTORY_METHOD = QconfObjectFactory.generate_complex_configuration
    OBJECT_NAME_KEY = None
    OBJECT_CLASS_NAME = 'ComplexConfiguration'
    OBJECT_CLASS_UGE_NAME = 'c'

    def __init__(self, qconf_executor):
        DictBasedObjectManager.__init__(self, qconf_executor)

    def add_object(self, pycl_object=None, data=None,
                   metadata=None, json_string=None):
        raise InvalidRequest('Complex configuration object cannot be added.')

    def delete_object(self):
        raise InvalidRequest('Complex configuration cannot be deleted.')

    def list_objects(self):
        raise InvalidRequest('This method is not supported for complex configuration.')

    def add_cattr(self, name, data):
        cconf = self.get_object('')
        cconf.check_attribute_data(name, data)
        if name in cconf.data:
            raise ObjectAlreadyExists('Complex attribute %s already exists.' % name)
        cconf.data[name] = data
        return self.replace_object(cconf)

    def modify_cattr(self, name, data):
        cconf = self.get_object('')
        if name in cconf.data:
            attr_data = cconf.data[name]
            attr_data.update(data)
        else:
            raise ObjectNotFound('Complex attribute %s does not exist.' % name)
        cconf.check_attribute_data(name, attr_data)
        return self.replace_object(cconf)

    def delete_cattr(self, name):
        cconf = self.get_object('')
        if name in cconf.data:
            del cconf.data[name]
        else:
            raise ObjectNotFound('Complex attribute %s does not exist.' % name)
        return self.replace_object(cconf)


#############################################################################
# Testing.
if __name__ == '__main__':
    pass



#!/usr/bin/env python
#
# ___INFO__MARK_BEGIN__
#######################################################################################
# Copyright 2016-2024 Altair Engineering Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#######################################################################################
# ___INFO__MARK_END__
#
import re
from uge.exceptions.object_not_found import ObjectNotFound
from uge.exceptions.object_already_exists import ObjectAlreadyExists
from uge.exceptions.invalid_request import InvalidRequest
from .list_based_object_manager import ListBasedObjectManager


class AdminHostManager(ListBasedObjectManager):
    QCONF_ERROR_REGEX_LIST = [
        (re.compile('.*already exists.*'), ObjectAlreadyExists),
        (re.compile('.*does not exist.*'), ObjectNotFound),
        (re.compile('.*may not remove.*'), InvalidRequest),
        (re.compile('.*resolving host.*'), ObjectNotFound),
        (re.compile('.*no admin host defined.*'), ObjectNotFound),
        (re.compile('.*denied: pseudo host.*'), InvalidRequest),
    ]

    OBJECT_NAME = 'admin host'
    OBJECT_CLASS_UGE_NAME = 'h'

    def __init__(self, qconf_executor):
        ListBasedObjectManager.__init__(self, qconf_executor)


#############################################################################
# Testing.
if __name__ == '__main__':
    pass



#!/usr/bin/env python
#
# ___INFO__MARK_BEGIN__
#######################################################################################
# Copyright 2016-2024 Altair Engineering Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#######################################################################################
# ___INFO__MARK_END__
#

from uge.exceptions import InvalidRequest

try:
    import UserList
except ImportError:
    import collections as UserList
from uge.exceptions.invalid_argument import InvalidArgument
from .qconf_object import QconfObject


class QconfDictList(QconfObject, UserList.UserList):
    """ This class encapsulates data and functionality common to all Qconf objects based on a list of dictionaries. """

    FIRST_KEY = None
    NAME_KEY = None
    USER_PROVIDED_KEYS = []
    REQUIRED_DATA_DEFAULTS = {}
    KEY_VALUE_DELIMITER = '='

    def __init__(self, data=None, metadata=None, json_string=None):
        """ 
        Class constructor. 

        :param data: Configuration data. If provided, it will override corresponding data from JSON string representation.
        :type data: dict

        :param metadata: Configuration metadata. If provided, it will override corresponding metadata from JSON string representation.
        :type metadata: dict

        :param json_string: Configuration JSON string representation.
        :type json_string: str

        :raises: **InvalidArgument** - in case metadata is not a dictionary, JSON string is not valid, or it does not represent a list of dictionaries object.
        """
        UserList.UserList.__init__(self)
        QconfObject.__init__(self, data=data, metadata=metadata, json_string=json_string)

    def check_input_data(self, data):
        if type(data) != list:
            raise InvalidArgument('Provided data is not a list: %s.' % str(data))
        for d in data:
            if type(d) != dict:
                raise InvalidArgument('List member is not a dictionary: %s.' % str(d))

    def update_with_required_data_defaults(self):
        """ 
        Updates list objects with default values for required data keys.

        :raises: **InvalidArgument** - in case object's data is not a list, or one of the list members is not a dictionary.
        """
        if type(self.data) != list:
            raise InvalidRequest('Data object is not a list: %s.' % str(self.data))
        for d in self.data:
            if type(d) != dict:
                raise InvalidArgument('List member is not a dictionary: %s.' % str(d))
            for (key, value) in list(self.get_required_data_defaults().items()):
                if key not in d:
                    d[key] = value

    def check_user_provided_keys(self):
        """ 
        Checks for presence of all data keys that must be provided by user.

        :raises: **InvalidRequest** - in case object's data is not a dictionary, or if any of the required keys are missing.
        """
        for d in self.data:
            if type(d) != dict:
                raise InvalidRequest('List member is not a dictionary: %s.' % str(d))
            for key in self.USER_PROVIDED_KEYS:
                if not d.get(key):
                    raise InvalidRequest('Input data %s is missing required object key: %s.' % (str(d), str(key)))

    def to_uge(self):
        """ 
        Converts object to string acceptable as input for UGE qconf command.

        :returns: Object's UGE-formatted string.
        """
        lines = ''
        for d in self.data:
            for (key, value) in list(d.items()):
                lines += '%s%s%s\n' % (key, self.KEY_VALUE_DELIMITER, self.py_to_uge(key, value))
        return lines

    def convert_data_to_uge_keywords(self, data):
        for d in data:
            for (key, value) in list(d.items()):
                d[key] = self.py_to_uge(key, value)

    def set_data_dict_list_from_qconf_output(self, qconf_output):
        data = self.to_dict_list(qconf_output)
        self.data = data

    def to_dict_list(self, input_string):
        lines = input_string.split('\n')
        dict_list = []
        object_data = {}
        # Parse lines until first object key is found, and then
        # create new dictionary object
        for line in lines:
            if not line:
                continue
            key_value = line.split(self.KEY_VALUE_DELIMITER)
            key = key_value[0]
            value = self.KEY_VALUE_DELIMITER.join(key_value[1:]).strip()
            if key == self.FIRST_KEY:
                object_data = {}
                dict_list.append(object_data)
            object_data[key] = self.uge_to_py(key, value)
        return dict_list


#############################################################################
# Testing.
if __name__ == '__main__':
    pass



#!/usr/bin/env python
#
# ___INFO__MARK_BEGIN__
#######################################################################################
# Copyright 2016-2024 Altair Engineering Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#######################################################################################
# ___INFO__MARK_END__
#

import logging
import os
import inspect


class TraceLogRecord(logging.LogRecord):
    """
    Our trace log record class for overriding call stack
    """

    def __init__(self, name, level, fn, lno, msg, args, exc_info, func):
        logging.LogRecord.__init__(
            self, name, level, fn, lno, msg, args, exc_info, func)

        self.pathname, self.lineno, self.func = self.__find_trace_caller()

        self.filename = os.path.basename(self.pathname)

    def __find_trace_caller(self):
        """
        Custom method for getting stack trace when using the trace method
        """

        f = inspect.currentframe()

        caller = f.f_back.f_back.f_back.f_back.f_back.f_back

        frameinfo = inspect.getframeinfo(caller)

        return frameinfo.filename, frameinfo.lineno, frameinfo.function



#!/usr/bin/env python
#
# ___INFO__MARK_BEGIN__
#######################################################################################
# Copyright 2016-2024 Altair Engineering Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#######################################################################################
# ___INFO__MARK_END__
#
import copy
from uge.log.log_manager import LogManager
from uge.exceptions.object_not_found import ObjectNotFound
from uge.exceptions.object_already_exists import ObjectAlreadyExists
from uge.exceptions.invalid_argument import InvalidArgument
from uge.exceptions.invalid_request import InvalidRequest
from uge.objects.qconf_object_factory import QconfObjectFactory
from uge.objects.qconf_object import QconfObject
from uge.objects.qconf_name_list import QconfNameList



class DictListBasedObjectManager(object):
    QCONF_ERROR_REGEX_LIST = [
    ]

    # Failure incorrectly classified as successful outcome
    QCONF_FAILURE_REGEX_LIST = []

    GENERATE_OBJECT_FACTORY_METHOD = None
    OBJECT_CLASS_NAME = None
    OBJECT_CLASS_UGE_NAME = None

    def __init__(self, qconf_executor):
        self.logger = LogManager.get_instance().get_logger(self.__class__.__name__)
        self.qconf_executor = qconf_executor

    def generate_object(self, data=None, metadata=None,
                        json_string=None, uge_version=None,
                        add_required_data=True):
        if not uge_version:
            uge_version = self.qconf_executor.get_uge_version()
        generated_object = self.GENERATE_OBJECT_FACTORY_METHOD(
            uge_version, data=data, metadata=metadata,
            json_string=json_string, add_required_data=add_required_data)
        return generated_object

    def __prepare_object(self, pycl_object=None, data=None,
                         metadata=None, json_string=None,
                         add_required_data=True):
        uge_version = self.qconf_executor.get_uge_version()
        data2 = copy.copy(data)
        metadata2 = copy.copy(metadata)
        # If pycl_object is provided, combine its metadata
        # with provided metadata
        if pycl_object is not None:
            generated_object = self.GENERATE_OBJECT_FACTORY_METHOD(uge_version, add_required_data=False)
            if str(type(pycl_object)) != str(type(generated_object)):
                raise InvalidArgument(
                    'The pycl_object argument must be an instance of %s.' % generated_object.__class__.__name__)
            data2 = copy.copy(pycl_object.data)
            if data:
                data2 = copy.copy(data)
            metadata2 = copy.copy(pycl_object.metadata)
            if metadata:
                metadata2.update(metadata)
        result_object = self.GENERATE_OBJECT_FACTORY_METHOD(
            uge_version, data=data2, metadata=metadata2,
            json_string=json_string, add_required_data=add_required_data)
        result_object.check_user_provided_keys()
        return result_object

    def verify_object_before_add(self, pycl_object):
        return

    def add_object(self, pycl_object=None, data=None,
                   metadata=None, json_string=None):
        new_object = self.__prepare_object(
            pycl_object=pycl_object, data=data,
            metadata=metadata, json_string=json_string)
        try:
            old_object = self.get_object()
            raise ObjectAlreadyExists('%s already exists.' % (self.OBJECT_CLASS_NAME))
        except ObjectNotFound as ex:
            # ok
            pass
        self.verify_object_before_add(new_object)
        self.qconf_executor.execute_qconf_with_object('-A%s' % self.OBJECT_CLASS_UGE_NAME, new_object,
                                                      self.QCONF_ERROR_REGEX_LIST)
        new_object.set_add_metadata()
        return new_object

    def verify_object_before_modify(self, pycl_object):
        return

    def modify_object(self, pycl_object=None, data=None,
                      metadata=None, json_string=None):
        generated_object = self.__prepare_object(
            pycl_object=pycl_object, data=data,
            metadata=metadata, json_string=json_string,
            add_required_data=True)
        updated_object = self.get_object()
        updated_object.data = generated_object.data
        self.verify_object_before_modify(updated_object)
        self.qconf_executor.execute_qconf_with_object('-M%s' % self.OBJECT_CLASS_UGE_NAME, updated_object,
                                                      self.QCONF_ERROR_REGEX_LIST)
        updated_object.set_modify_metadata()
        return updated_object

    def replace_object(self, updated_object):
        self.qconf_executor.execute_qconf_with_object('-M%s' % self.OBJECT_CLASS_UGE_NAME, updated_object,
                                                      self.QCONF_ERROR_REGEX_LIST)
        updated_object.set_modify_metadata()
        return updated_object

    def get_object(self):
        uge_version = self.qconf_executor.get_uge_version()
        retrieved_object = self.GENERATE_OBJECT_FACTORY_METHOD(uge_version, add_required_data=False)
        qconf_output = self.qconf_executor.execute_qconf('-s%s' % (self.OBJECT_CLASS_UGE_NAME),
                                                         self.QCONF_ERROR_REGEX_LIST,
                                                         failure_regex_list=self.QCONF_FAILURE_REGEX_LIST).get_stdout()
        retrieved_object.set_data_dict_list_from_qconf_output(qconf_output)
        return retrieved_object

    def verify_object_before_delete(self, pycl_object):
        return

    def delete_object(self):
        deleted_object = self.get_object()
        self.verify_object_before_delete(deleted_object)
        self.qconf_executor.execute_qconf('-d%s' % (self.OBJECT_CLASS_UGE_NAME), self.QCONF_ERROR_REGEX_LIST)


#############################################################################
# Testing.
if __name__ == '__main__':
    pass



#!/usr/bin/env python
# 
# ___INFO__MARK_BEGIN__
#######################################################################################
# Copyright 2016-2024 Altair Engineering Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#######################################################################################
# ___INFO__MARK_END__
# 
import types
from .utils import needs_uge
from .utils import generate_random_string
from .utils import create_config_file

from uge.api.qconf_api import QconfApi
from uge.exceptions.object_not_found import ObjectNotFound
from uge.exceptions.object_already_exists import ObjectAlreadyExists

create_config_file()
API = QconfApi()
RQS_NAME = 'rqs%s' % generate_random_string(6)


@needs_uge
def test_object_not_found():
    try:
        rqs = API.get_rqs('__non_existent_rqs__')
        assert (False)
    except ObjectNotFound as ex:
        # ok
        pass


def test_generate_rqs():
    rqs = API.generate_rqs(RQS_NAME)
    assert (rqs.data['name'] == RQS_NAME)


def test_add_rqs():
    rqs_list = API.list_rqss()
    rqs = API.add_rqs(name=RQS_NAME)
    assert (rqs.data['name'] == RQS_NAME)
    rqs_list2 = API.list_rqss()
    assert (len(rqs_list2) == len(rqs_list) + 1)
    assert (rqs_list2.count(RQS_NAME) == 1)


def test_list_rqss():
    rqs_list = API.list_rqss()
    assert (rqs_list is not None)


def test_object_already_exists():
    try:
        rqs = API.add_rqs(name=RQS_NAME)
        assert (False)
    except ObjectAlreadyExists as ex:
        # ok
        pass


def test_get_rqs():
    rqs = API.get_rqs(RQS_NAME)
    assert (rqs.data['name'] == RQS_NAME)


def test_generate_rqs_from_json():
    rqs = API.get_rqs(RQS_NAME)
    json = rqs.to_json()
    rqs2 = API.generate_object(json)
    assert (rqs2.__class__.__name__ == rqs.__class__.__name__)
    for key in list(rqs.data.keys()):
        v = rqs.data[key]
        v2 = rqs2.data[key]
        if type(v) == list:
            assert (len(v) == len(v2))
            for s in v:
                assert (v2.count(s) == 1)
        elif type(v) == dict:
            for key in list(v.keys()):
                assert (str(v[key]) == str(v2[key]))
        else:
            assert (str(v) == str(v2))


def test_modify_rqs():
    rqs = API.get_rqs(RQS_NAME)
    enabled = rqs.data['enabled']
    rqs = API.modify_rqs(name=RQS_NAME, data={'enabled': not enabled})
    enabled2 = rqs.data['enabled']
    assert (enabled2 == (not enabled))


def test_delete_rqs():
    rqs_list = API.list_rqss()
    API.delete_rqs(RQS_NAME)
    rqs_list2 = API.list_rqss()
    assert (len(rqs_list2) == len(rqs_list) - 1)
    assert (rqs_list2.count(RQS_NAME) == 0)



#!/usr/bin/env python
#
# ___INFO__MARK_BEGIN__
#######################################################################################
# Copyright 2016-2024 Altair Engineering Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#######################################################################################
# ___INFO__MARK_END__
#
from __future__ import print_function
import abc
import logging
import os
import sys
from optparse import OptionGroup
from optparse import OptionParser

from uge.config.config_manager import ConfigManager
from uge.exceptions.invalid_argument import InvalidArgument
from uge.exceptions.qconf_exception import QconfException
from uge.log.log_manager import LogManager
# compatible with Python 2 *and* 3:
ABC = abc.ABCMeta('ABC', (object,), {'__slots__': ()})

class QconfCli(ABC):
    """ Base qconf command line interface class. """

    def __init__(self, valid_arg_count=0):
        """ 
        Class constructor.

        :param valid_arg_count: Number of allowed positional arguments (default: 0).
        :type valid_arg_count: int
        """
        self.logger = LogManager.get_instance().get_logger(self.__class__.__name__)
        self.parser = OptionParser(add_help_option=False)
        self.options = {}
        self.args = []
        self.valid_arg_count = valid_arg_count
        self.option_group_dict = {}

        common_group = 'Common Options'
        self.add_option_group(common_group, None)

        self.add_option_to_group(common_group, '-h', '--help', action='help', help='Show this help message and exit.')
        self.add_option_to_group(common_group, '-?', '', action='help', help='Show this help message and exit.')

        self.add_option_to_group(common_group, '-v', '', action='store_true', dest='cmd_version', default=False,
                                 help='Print version and exit.')

        self.add_option_to_group(common_group, '-d', '--debug', dest='console_log_level',
                                 help='Set debug level; valid values are: critical, error, warning, info, debug')

    def add_option(self, *args, **kwargs):
        """ 
        Add CLI option. 
        """
        self.parser.add_option(*args, **kwargs)

    def add_option_to_group(self, group_name, *args, **kwargs):
        """
        Add option to the given group.
        Group should be created using add_option_group().

        :param group_name: Group name.
        :type group_name: str
        """
        group = self.option_group_dict.get(group_name)
        group.add_option(*args, **kwargs)

    def add_option_group(self, group_name, desc):
        """ 
        Add option group. 

        :param group_name: Group name.
        :type group_name: str
        """
        group = OptionGroup(self.parser, group_name, desc)
        self.parser.add_option_group(group)
        self.option_group_dict[group_name] = group

    def parse_args(self, usage=None):
        """ 
        Parse command arguments. 

        :param usage: Command usage.
        :type usage: str
        """
        if usage:
            self.parser.usage = usage

        try:
            (self.options, self.args) = self.parser.parse_args()
        except SystemExit as rc:
            sys.stdout.flush()
            sys.stderr.flush()
            os._exit(int(str(rc)))

        if self.valid_arg_count < len(self.args):
            # Postitional args are not enabled and we have some
            msg = "Invalid Argument(s):"
            for arg in self.args[self.valid_arg_count:]:
                msg += " " + arg
            raise InvalidArgument(msg)

        opt_dict = self.options.__dict__
        if opt_dict.get('cmd_version'):
            print('%s version: %s' % (os.path.basename(sys.argv[0]), ConfigManager.get_instance().get_version()))
            os._exit(0)

        # Log level.
        console_log_level = opt_dict.get('console_log_level', None)
        if console_log_level:
            LogManager.get_instance().set_console_log_level(console_log_level)

        # Check input arguments.
        self.check_input_args()
        return (self.options, self.args)

    def usage(self, s=None):
        """ Print the help provided by optparse. """

        if s: print('Error:', s, '\n', file=sys.stderr)
        self.parser.print_help()
        os._exit(1)

    def get_options(self):
        """ Returns the command line options. """
        return self.options

    def get_n_args(self):
        """ Returns the number of command line arguments. """
        return len(self.args)

    def get_args(self):
        """ Returns the command line argument list. """
        return self.args

    def get_arg(self, i):
        """ Returns the i-th command line argument. """
        return self.args[i]

    @abc.abstractmethod
    def run_command(self):
        """ This method must be implemented by the derived class. """

    def check_input_args(self):
        """ 
        This method should verify required arguments in the derived class.
        """

    def run(self):
        """
        Run command. This method simply invokes run_command() and handles
        any exceptions.
        """
        try:
            self.run_command()
        except QconfException as ex:
            if self.logger.level < logging.INFO:
                self.logger.exception('%s' % ex)
            print('%s' % ex.get_error_message())
            raise SystemExit(ex.get_error_code())
        except SystemExit as ex:
            raise
        except Exception as ex:
            self.logger.exception('%s' % ex)
            print('%s' % ex)
            raise SystemExit(-1)


#############################################################################
# Testing.
if __name__ == '__main__':
    cli = QconfCli()
    cli.add_option("-f", "--file", dest="filename",
                   help="write report to FILE", metavar="FILE")
    cli.add_option("-q", "--quiet",
                   action="store_false", dest="verbose", default=True,
                   help="don't print status messages to stdout")
    (options, args) = cli.parse_args()
    print('OPTIONS: ', options)
    print('ARGS: ', args)

    print('OPTIONS: ', cli.get_options())
    print('ARGS: ', cli.get_args())
    print('options.filename', options.filename)
    print('cli.getOptions().filename', cli.get_options().filename)
    o = cli.get_options()
    print('o.filename', o.filename)

    print('cli.get_args()', cli.get_args())
    print('len(cli.get_args())', len(cli.get_args()))

    for a in cli.get_args():
        print('arg', a)

    first_arg = cli.get_arg(0)
    print('first_arg', first_arg)

    second_arg = cli.get_arg(1)
    print('second_arg', second_arg)

    try:
        third_arg = cli.get_arg(2)
        print('third_arg', third_arg)
    except:
        print('no third arg')



#!/usr/bin/env python
#
# ___INFO__MARK_BEGIN__
#######################################################################################
# Copyright 2016-2024 Altair Engineering Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#######################################################################################
# ___INFO__MARK_END__
#
import re
from uge.exceptions.object_not_found import ObjectNotFound
from uge.exceptions.invalid_request import InvalidRequest
from uge.exceptions.invalid_argument import InvalidArgument
from uge.objects.qconf_object_factory import QconfObjectFactory
from .dict_list_based_object_manager import DictListBasedObjectManager

class ShareTreeManager(DictListBasedObjectManager):

    QCONF_ERROR_REGEX_LIST = [
        (re.compile('.*no sharetree element.*'),ObjectNotFound),
        (re.compile('.*denied: share tree contains reference to unknown user/project.*'),InvalidRequest),
        (re.compile('.*Unable to locate.*'),ObjectNotFound),
        (re.compile('.share value must be positive.*'),InvalidArgument),
    ]

    QCONF_ERROR_SSTREE_REGEX_LIST = [
        (re.compile('.*no sharetree element.*'),ObjectNotFound),
    ]

    GENERATE_OBJECT_FACTORY_METHOD = QconfObjectFactory.generate_share_tree
    OBJECT_NAME_KEY = None
    OBJECT_CLASS_NAME = 'ShareTree'
    OBJECT_CLASS_UGE_NAME = 'stree'

    def __init__(self, qconf_executor):
        DictListBasedObjectManager.__init__(self, qconf_executor)

    def add_stnode(self, path, shares):
        try:
            share_value = int(shares)
        except ValueError as ex:
            raise InvalidArgument(exception=ex)
        self.qconf_executor.execute_qconf('-astnode %s=%s' % (path, shares), self.QCONF_ERROR_REGEX_LIST)
        return self.get_object()

    def delete_stnode(self, path):
        self.qconf_executor.execute_qconf('-dstnode %s' % (path), self.QCONF_ERROR_REGEX_LIST)
        return self.get_object()

    def object_exists(self):
        try:
            self.qconf_executor.execute_qconf('-sstree', self.QCONF_ERROR_SSTREE_REGEX_LIST)
        except ObjectNotFound:
            return False
        return True

    # The following methods implement behavior where no share tree
    # is equivalent to empty list
    def get_object_if_exists(self):
        # Return empty list if share is not there
        if self.object_exists():
            stree = self.get_object()
        else:
            stree = self.generate_object(data=[], add_required_data=False)
        return stree

    def delete_object_if_exists(self):
        # Do not throw exception if share tree is not there
        if self.object_exists():
            self.delete_object()

    def modify_or_add_object(self, pycl_object=None, data=None,
                      metadata=None, json_string=None):
        # If empty list is provided, delete share tree
        if type(data) == list and not len(data):
            self.delete_object_if_exists()
            return self.generate_object(data=[], add_required_data=False)
        # If share tree exists, modify it
        # If share tree is not there, add it.
        if self.object_exists():
            stree = self.modify_object(pycl_object=pycl_object, data=data, metadata=metadata, json_string=json_string)
        else:
            stree = self.add_object(pycl_object=pycl_object, data=data, metadata=metadata, json_string=json_string)
        return stree

#############################################################################
# Testing.
if __name__ == '__main__':
    pass



#!/usr/bin/env python
#
# ___INFO__MARK_BEGIN__
#######################################################################################
# Copyright 2016-2024 Altair Engineering Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#######################################################################################
# ___INFO__MARK_END__
#

import tempfile

from nose import SkipTest

from .utils import needs_uge
from .utils import generate_random_string
from .utils import create_config_file
from .utils import load_values

from uge.api.qconf_api import QconfApi
from uge.config.config_manager import ConfigManager
from uge.log.log_manager import LogManager
from uge.exceptions.object_not_found import ObjectNotFound
from uge.exceptions.object_already_exists import ObjectAlreadyExists

create_config_file()
API = QconfApi()
HOST_NAME = '%s' % generate_random_string(6)
CONFIG_MANAGER = ConfigManager.get_instance()
LOG_MANAGER = LogManager.get_instance()
VALUES_DICT = load_values('test_values.json')
print(VALUES_DICT)


@needs_uge
def test_generate_ehost():
    h = API.generate_ehost(HOST_NAME)
    assert (h.data['hostname'] == HOST_NAME)


def test_list_ehosts():
    hl = API.list_ehosts()
    assert (hl is not None)


def test_object_already_exists():
    try:
        hl = API.list_ehosts()
        if len(hl):
            h = API.add_ehost(name=hl[0])
            assert (False)
        else:
            raise SkipTest('There are no configured UGE execution hosts.')
    except ObjectAlreadyExists as ex:
        # ok
        pass


def test_get_ehost():
    hl = API.list_ehosts()
    if len(hl):
        h = API.get_ehost(hl[0])
        assert (h.data['hostname'] == hl[0])
    else:
        raise SkipTest('There are no configured UGE execution hosts.')


def test_modify_ehost():
    hl = API.list_ehosts()
    if len(hl):
        h = API.get_ehost(hl[0])
        original_complex_values = h.data['complex_values']
        new_complex_values = ['slots=100']
        h2 = API.modify_ehost(name=hl[0], data={'complex_values': new_complex_values})
        output_string = '%s' % h2.data['complex_values']
        input_string = '%s' % new_complex_values
        assert (input_string == output_string)
        h3 = API.modify_ehost(name=hl[0], data={'complex_values': original_complex_values})
        output_string = '%s' % h3.data['complex_values']
        input_string = '%s' % original_complex_values
        assert (input_string == output_string)
    else:
        raise SkipTest('There are no configured UGE execution hosts.')


def test_get_ehosts():
    ehl = API.list_ehosts()
    ehosts = API.get_ehosts()
    for eh in ehosts:
        print("#############################################")
        print(eh.to_uge())
        assert (eh.data['hostname'] in ehl)


def test_write_ehosts():
    try:
        tdir = tempfile.mkdtemp()
        print("*************************** " + tdir)
        host_names = VALUES_DICT['host_names']
        ehosts = API.get_ehosts()
        for eh in ehosts:
            print("Before #############################################")
            print(eh.to_uge())

        new_hosts = []
        for name in host_names:
            neh = API.generate_ehost(name=name)
            new_hosts.append(neh)
        API.mk_ehosts_dir(tdir)
        API.write_ehosts(new_hosts, tdir)
        API.add_ehosts_from_dir(tdir)
        API.modify_ehosts_from_dir(tdir)
        ehosts = API.get_ehosts()
        for eh in ehosts:
            print("After #############################################")
            print(eh.to_uge())

        ehosts = API.list_ehosts()
        for name in host_names:
            assert (name in ehosts)
            print("execution host found: " + name)

    finally:
        API.delete_ehosts(host_names)
        API.rm_ehosts_dir(tdir)


def test_add_ehosts():
    try:
        new_hosts = []
        host_names = VALUES_DICT['host_names']
        for name in host_names:
            neh = API.generate_ehost(name=name)
            new_hosts.append(neh)

        # print all execution hosts currently in the cluster
        ehosts = API.get_ehosts()
        for eh in ehosts:
            print("Before #############################################")
            print(eh.to_uge())

        # add execution hosts
        API.add_ehosts(new_hosts)

        # print all execution hosts currently in the cluster
        ehosts = API.get_ehosts()
        for eh in ehosts:
            print("After #############################################")
            print(eh.to_uge())

        # check that hosts have been added
        ehosts = API.list_ehosts()
        for name in host_names:
            assert (name in ehosts)
            print("execution host found: " + name)

    finally:
        API.delete_ehosts(host_names)



#!/usr/bin/env python
#
# ___INFO__MARK_BEGIN__
#######################################################################################
# Copyright 2016-2024 Altair Engineering Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#######################################################################################
# ___INFO__MARK_END__
#

import tempfile

from .utils import needs_uge
from .utils import generate_random_string
from .utils import generate_random_string_list
from .utils import create_config_file
from .utils import load_values

from uge.api.qconf_api import QconfApi
from uge.config.config_manager import ConfigManager
from uge.log.log_manager import LogManager
from uge.exceptions.object_not_found import ObjectNotFound
from uge.exceptions.object_already_exists import ObjectAlreadyExists

create_config_file()
API = QconfApi()
ACL_NAME = 'acl.%s' % generate_random_string(6)
USER1_NAME = 'user.%s' % generate_random_string(6)
USER2_NAME = 'user.%s' % generate_random_string(6)
CONFIG_MANAGER = ConfigManager.get_instance()
LOG_MANAGER = LogManager.get_instance()
VALUES_DICT = load_values('test_values.json')
print(VALUES_DICT)


@needs_uge
def test_object_not_found():
    try:
        acl = API.get_acl('__non_existent_acl__')
        assert(False)
    except ObjectNotFound as ex:
        # ok
        pass


def test_generate_acl():
    acl = API.generate_acl(ACL_NAME)
    assert(acl.data['name'] == ACL_NAME)


def test_add_acl():
    try:
        acl_list = API.list_acls()
    except ObjectNotFound as ex:
        # no acls defined
        acl_list = []
    acl = API.add_acl(name=ACL_NAME)
    assert(acl.data['name'] == ACL_NAME)
    acl_list2 = API.list_acls()
    assert(len(acl_list2) == len(acl_list) + 1)
    assert(acl_list2.count(ACL_NAME) == 1)


def test_list_acls():
    acl_list = API.list_acls()
    assert(acl_list is not None)


def test_object_already_exists():
    try:
        acl = API.add_acl(name=ACL_NAME)
        assert(False)
    except ObjectAlreadyExists as ex:
        # ok
        pass


def test_get_acl():
    acl = API.get_acl(ACL_NAME)
    assert(acl.data['name'] == ACL_NAME)


def test_generate_acl_from_json():
    acl = API.get_acl(ACL_NAME)
    assert(acl.data['name'] == ACL_NAME)
    json = acl.to_json()
    acl2 = API.generate_object(json)
    assert(acl2.__class__.__name__ == acl.__class__.__name__)
    for key in list(acl.data.keys()):
        v = acl.data[key]
        v2 = acl2.data[key]
        assert(str(v) == str(v2))


def test_modify_acl():
    acl = API.get_acl(ACL_NAME)
    acl = API.modify_acl(name=ACL_NAME, data={'entries' : [USER1_NAME, USER2_NAME]})
    entries = acl.data['entries']
    assert(len(entries) == 2)


def test_get_acls():
    acll = API.list_acls()
    acls = API.get_acls()
    for acl in acls:
        print("#############################################")
        print(acl.to_uge())
        assert (acl.data['name'] in acll)


def test_write_acls():
    try:
        tdir = tempfile.mkdtemp()
        print("*************************** " + tdir)
        acl_names = VALUES_DICT['acl_names']
        acls = API.get_acls()
        for acl in acls:
            print("Before #############################################")
            print(acl.to_uge())

        new_acls = []
        for name in acl_names:
            nacl = API.generate_acl(name=name)
            new_acls.append(nacl)
        API.mk_acls_dir(tdir)
        API.write_acls(new_acls, tdir)
        API.add_acls_from_dir(tdir)
        API.modify_acls_from_dir(tdir)
        acls = API.get_acls()
        for acl in acls:
            print("After #############################################")
            print(acl.to_uge())

        acls = API.list_acls()
        for name in acl_names:
            assert (name in acls)
            print("acl found: " + name)

    finally:
        API.delete_acls_from_dir(tdir)
        API.rm_acls_dir(tdir)


def test_add_acls():
    try:
        new_acls = []
        acl_names = VALUES_DICT['acl_names']
        for name in acl_names:
            nacl = API.generate_acl(name=name)
            new_acls.append(nacl)

        # print all acls currently in the cluster
        acls = API.get_acls()
        for acl in acls:
            print("Before #############################################")
            print(acl.to_uge())

        # add acls
        API.add_acls(new_acls)
        API.modify_acls(new_acls)

        # print all acls currently in the cluster
        acls = API.get_acls()
        for acl in acls:
            print("After #############################################")
            print(acl.to_uge())

        # check that cals have been added
        acls = API.list_acls()
        for name in acl_names:
            assert (name in acls)
            print("acl found: " + name)

    finally:
        API.delete_acls(new_acls)


def test_add_users_to_acl_and_delete_users_from_acl():
    acl_list = API.list_acls()
    n_acls = 3
    n_users = 3
    acl_name_list = generate_random_string_list(n_strings=3, string_length=6, delimiter=',', string_prefix='acl.')
    user_name_list = generate_random_string_list(n_strings=3, string_length=6, delimiter=',', string_prefix='user.')
    new_acls = API.add_users_to_acls(user_name_list, acl_name_list)
    assert(len(new_acls) == n_acls)
    for i in range (0,n_acls):
        assert(len(new_acls[i].data['entries']) == n_users)
    acl_list2 = API.list_acls()
    assert(len(acl_list2) == len(acl_list) + n_acls)
    modified_acls = API.delete_users_from_acls(user_name_list, acl_name_list)
    for i in range (0,n_acls):
        assert(modified_acls[i].data['entries'] is None)
    for i in range (0,n_acls):
        API.delete_acl(modified_acls[i].data['name'])
    acl_list3 = API.list_acls()
    assert(len(acl_list3) == len(acl_list))


def test_delete_acl():
    acl_list = API.list_acls()
    API.delete_acl(ACL_NAME)
    try:
        acl_list2 = API.list_acls()
    except ObjectNotFound as ex:
        # no acls defined
        acl_list2 = []
    assert(len(acl_list2) == len(acl_list) - 1)
    assert(acl_list2.count(ACL_NAME) == 0)



#!/usr/bin/env python
#
# ___INFO__MARK_BEGIN__
#######################################################################################
# Copyright 2016-2024 Altair Engineering Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#######################################################################################
# ___INFO__MARK_END__
#
import re
from uge.exceptions.object_not_found import ObjectNotFound
from uge.exceptions.object_already_exists import ObjectAlreadyExists
from uge.exceptions.invalid_request import InvalidRequest
from uge.exceptions.invalid_argument import InvalidArgument
from uge.objects.qconf_object_factory import QconfObjectFactory
from .dict_based_object_manager import DictBasedObjectManager


class AccessListManager(DictBasedObjectManager):
    QCONF_ERROR_REGEX_LIST = [
        (re.compile('.*is still referenced in.*'), InvalidRequest),
        (re.compile('.*not allowed to set.*'), InvalidRequest),
        (re.compile('.*unknown specifier.*'), InvalidRequest),
        (re.compile('.*does not exist.*'), ObjectNotFound),
        (re.compile('.*doesn\'t exist.*'), ObjectNotFound),
        (re.compile('.* no list_name.*'), ObjectNotFound),
        (re.compile('.*is already in access list.*'), ObjectAlreadyExists),
        (re.compile('.*is not in access list.*'), ObjectNotFound),
    ]

    # Failure incorrectly classified as successful outcome
    QCONF_FAILURE_REGEX_LIST = []

    GENERATE_OBJECT_FACTORY_METHOD = QconfObjectFactory.generate_access_list
    OBJECT_NAME_KEY = 'name'
    OBJECT_CLASS_NAME = 'AccessList'
    OBJECT_CLASS_UGE_NAME = 'u'
    OBJECT_CLASS_UGE_LIST_DETAILS_NAME = 'ld'

    def __init__(self, qconf_executor):
        DictBasedObjectManager.__init__(self, qconf_executor)

    def delete_object(self, name):
        deleted_object = self.get_object(name)
        self.qconf_executor.execute_qconf('-dul %s' % (name), self.QCONF_ERROR_REGEX_LIST)

    def __check_and_prepare_input(self, input_value, input_arg_name):
        if type(input_value) == bytes or type(input_value) == str:
            if input_value.find(' ') >= 0:
                raise InvalidArgument(
                    'Value for argument %s must be provided either as a comma-separated string list, or as a python '
                    'list of strings.' % input_arg_name)
            return input_value
        elif type(input_value) == list:
            input_value = ','.join(input_value)
            return input_value
        else:
            raise InvalidArgument(
                'Input value must be provided either as a comma-separated string list, or as a python list of strings '
                '(%s input: %s).' % (input_arg_name, input_value))

    def add_users_to_acls(self, user_names, access_list_names):
        user_name_list = self.__check_and_prepare_input(user_names, 'user_names')
        acl_name_list = self.__check_and_prepare_input(access_list_names, 'access_list_names')
        self.qconf_executor.execute_qconf('-au %s %s' % (user_name_list, acl_name_list), self.QCONF_ERROR_REGEX_LIST)
        acl_list = []
        for acl_name in acl_name_list.split(','):
            acl = self.get_object(acl_name)
            acl_list.append(acl)
        return acl_list

    def delete_users_from_acls(self, user_names, access_list_names):
        user_name_list = self.__check_and_prepare_input(user_names, 'user_names')
        acl_name_list = self.__check_and_prepare_input(access_list_names, 'access_list_names')
        self.qconf_executor.execute_qconf('-du %s %s' % (user_name_list, acl_name_list), self.QCONF_ERROR_REGEX_LIST)
        acl_list = []
        for acl_name in acl_name_list.split(','):
            acl = self.get_object(acl_name)
            acl_list.append(acl)
        return acl_list

    def get_bulk_dump_filename(self, object):
        return 'conf_api_dump_' + object.data['name']

#############################################################################
# Testing.
if __name__ == '__main__':
    pass



#!/usr/bin/env python
#
# ___INFO__MARK_BEGIN__
#######################################################################################
# Copyright 2016-2024 Altair Engineering Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#######################################################################################
# ___INFO__MARK_END__
#

import re
import logging

from uge.log.trace_logger import TraceLogger


class LoggerFactory(object):
    def __init__(self, logger_expressions=None):
        """ Constructor...take in a list of regular expressions
            to match loggers against """
        self.expressions = []
        self.forced_level = logging.CRITICAL
        self.logger = logging.getLogger()
        if logger_expressions is not None:
            self.parse_expressions(logger_expressions)

    def get_logger(self, name):
        """ Get a logger by name and set its level accordingly """
        logger = logging.getLogger(name)
        logger.setLevel(self.get_level(name))
        return logger

    def get_level(self, name):
        """ Search through the list of expressions and find the level
            that matches a specific name.  If there isn't a match fall
            through to the next in the heirarchy """
        level = logging.NOTSET
        # Iterate in reverse as the last is most significant
        for exp in reversed(self.expressions):
            pattern, level = exp
            # If we return non None its a match
            if not None == pattern.match(name):
                # If we are forcing to a less severe level than the one we
                # matched we must override the level
                if level >= self.forced_level:
                    level = self.forced_level
                break

        return level

    def force_level(self, level):
        """ Force all loggers to at least a specific level """
        self.forced_level = level
        self.logger.setLevel(level)
        # self.logger.\
        #    log(TraceLogger.TRACE, 'Forced all loggers to %s' % level)

    def parse_expressions(self, expressions):
        """ Parse a list of logger matching expressions of the form
            <regex>=<log-level>.  Place the compiled regex's and levels
            in the expressions attribute. """
        lines = expressions.split('\n')
        for line in lines:
            try:
                # Use the right split so we can have '='s in the regex
                regex, level = line.rsplit('=', 1)
                pattern = re.compile(regex)
                # results = (pattern, logging.getLevelName(level.upper()))
                results = (pattern, logging._checkLevel(level.upper()))

                self.logger.log(
                    TraceLogger.TRACE,
                    'Appending %s:%s to logger level expressions' % (
                        results[0], results[1]))

                self.expressions.append(results)
            except Exception as ex:
                self.logger.error('Parser error in log configuration file: %s' % line)
                self.logger.exception(ex)



#!/usr/bin/env python
#
# ___INFO__MARK_BEGIN__
#######################################################################################
# Copyright 2016-2024 Altair Engineering Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#######################################################################################
# ___INFO__MARK_END__
#
import imp
import json
import re

from uge.exceptions.ar_exception import AdvanceReservationException
from uge.exceptions.invalid_request import InvalidRequest
from .uge_release_object_map import UGE_RELEASE_OBJECT_MAP


class AdvanceReservationObjectFactory(object):

    @classmethod
    def __get_object_base_module_name(cls, class_name):
        # This method relies on convention:
        #     ResourceQuotaSet=>resource_quota_set
        base_module_name = class_name[0].lower()
        for letter in class_name[1:]:
            if letter.isupper():
                base_module_name += '_%s' % letter.lower()
            else:
                base_module_name += letter
        return base_module_name

    @classmethod
    def __get_object_class_from_uge_version(cls, uge_version, class_name, base_module_name=None):
        if not uge_version:
            raise InvalidRequest('Cannot generate %s object: UGE version must be specified.' % class_name)
        uge_version = re.search(r'\d+.\d+.\d+', uge_version).group(0)
        if uge_version not in UGE_RELEASE_OBJECT_MAP:
            raise AdvanceReservationException('Unsupported UGE version: %s.' % uge_version)
        release_map = UGE_RELEASE_OBJECT_MAP.get(uge_version)
        object_version = release_map.get(class_name)
        return cls.__get_object_class_from_object_version(object_version, class_name, base_module_name)

    @classmethod
    def __get_object_class_from_object_version(cls, object_version, class_name, base_module_name=None):
        if not object_version:
            raise InvalidRequest('Object version not supplied for class %s.' % (class_name))

        if base_module_name is None:
            base_module_name = cls.__get_object_base_module_name(class_name)

        module_name = ('%s_v%s' % (base_module_name, object_version)).replace('.', '_')
        module_file = '%s/%s.py' % ('/'.join(__file__.split('/')[:-1]), module_name)
        module = imp.load_source('uge.objects.%s' % module_name, module_file)
        object_class = getattr(module, class_name)
        return object_class

    @classmethod
    def __generate_object(cls, object_class, name, data, metadata, json_string, add_required_data):
        generated_object = object_class(name=name, data=data, metadata=metadata, json_string=json_string)
        if add_required_data:
            generated_object.update_with_required_data_defaults()
        return generated_object

    @classmethod
    def generate_object(cls, json_string, target_uge_version=None):
        object_dict = json.loads(json_string)
        class_name = object_dict['object_class']
        object_version = object_dict['object_version']
        object_class = cls.__get_object_class_from_object_version(object_version, class_name)
        generated_object = object_class(json_string=json_string)
        generated_object.update_with_required_data_defaults()
        if target_uge_version:
            object_class = cls.__get_object_class_from_uge_version(target_uge_version, class_name)
            generated_object = object_class(json_string=generated_object.to_json())
            generated_object.update_with_required_data_defaults()
        return generated_object

    @classmethod
    def generate_advance_reservation(cls, uge_version, name=None, data=None, metadata=None, json_string=None,
                                     add_required_data=True):
        advance_reservation_class = cls.__get_object_class_from_uge_version(uge_version, 'AdvanceReservation')
        advance_reservation = advance_reservation_class(name=name, data=data, metadata=metadata,
                                                        json_string=json_string)
        if add_required_data:
            advance_reservation.update_with_required_data_defaults()
        return advance_reservation


#############################################################################
# Testing.
if __name__ == '__main__':
    print(__file__)
    module_file = '%s/%s.py' % ('/'.join(__file__.split('/')[:-1]), 'advance_reservation_v1_0')
    print(module_file)
    module = imp.load_source('uge.objects.advance_reservation_v1_0', module_file)
    print(module)
    print(getattr(module, 'AdvanceReservation'))
    ar = AdvanceReservationObjectFactory.generate_advance_reservation('8.6.0')
    print(ar)
    print(ar.VERSION)



#!/usr/bin/env python
# 
# ___INFO__MARK_BEGIN__
#######################################################################################
# Copyright 2016-2024 Altair Engineering Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#######################################################################################
# ___INFO__MARK_END__
# 
import types
from .utils import needs_uge
from .utils import generate_random_string
from .utils import create_config_file

from uge.api.qconf_api import QconfApi

create_config_file()
API = QconfApi()


@needs_uge
def test_generate_sconf():
    sconf = API.generate_sconf()
    assert ('schedule_interval' in sconf.data)


def test_get_sconf():
    sconf = API.get_sconf()
    assert ('schedule_interval' in sconf.data)


def test_generate_sconf_from_json():
    sconf = API.get_sconf()
    json = sconf.to_json()
    sconf2 = API.generate_object(json)
    assert (sconf2.__class__.__name__ == sconf.__class__.__name__)
    for key in list(sconf.data.keys()):
        v = sconf.data[key]
        v2 = sconf2.data[key]
        if type(v) == list:
            assert (len(v) == len(v2))
            for s in v:
                assert (v2.count(s) == 1)
        elif type(v) == dict:
            for key in list(v.keys()):
                assert (str(v[key]) == str(v2[key]))
        else:
            assert (str(v) == str(v2))


def test_modify_sconf():
    sconf = API.get_sconf()
    weight_user = sconf.data['weight_user']
    sconf = API.modify_sconf(data={'weight_user': weight_user * 2})
    weight_user2 = sconf.data['weight_user']
    assert (weight_user2 == weight_user * 2)
    sconf = API.modify_sconf(data={'weight_user': weight_user})
    weight_user3 = sconf.data['weight_user']
    assert (weight_user3 == weight_user)



#!/usr/bin/env python
#
# ___INFO__MARK_BEGIN__
#######################################################################################
# Copyright 2016-2024 Altair Engineering Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#######################################################################################
# ___INFO__MARK_END__
#
from .qconf_object import QconfObject


class SchedulerConfiguration(QconfObject):
    """ This class encapsulates UGE scheduler configuration object. """

    #: Object version.
    VERSION = '1.0'

    #: Object name key.
    NAME_KEY = None

    #: Object keys that must be provided by user.
    USER_PROVIDED_KEYS = []

    #: Default values for required data keys.
    REQUIRED_DATA_DEFAULTS = {
        'algorithm': 'default',
        'schedule_interval': '0:0:15',
        'maxujobs': 0,
        'queue_sort_method': 'load',
        'job_load_adjustments': 'np_load_avg=0.50',
        'load_adjustment_decay_time': '0:7:30',
        'load_formula': 'np_load_avg',
        'schedd_job_info': False,
        'flush_submit_sec': 1,
        'flush_finish_sec': 1,
        'params': None,
        'reprioritize_interval': '0:0:0',
        'halftime': 168,
        'usage_weight_list': ['wallclock=0.000000', 'cpu=1.000000', 'mem=0.000000', 'io=0.000000'],
        'compensation_factor': 5.0,
        'weight_user': 0.25,
        'weight_project': 0.25,
        'weight_department': 0.25,
        'weight_job': 0.25,
        'weight_tickets_functional': 0,
        'weight_tickets_share': 0,
        'share_override_tickets': True,
        'share_functional_shares': True,
        'max_functional_jobs_to_schedule': 200,
        'report_pjob_tickets': True,
        'max_pending_tasks_per_job': 50,
        'halflife_decay_list': None,
        'policy_hierarchy': 'OFS',
        'weight_ticket': 0.01,
        'weight_waiting_time': 0.0,
        'weight_deadline': 3600000.0,
        'weight_urgency': 0.1,
        'weight_priority': 1.0,
        'fair_urgency_list': None,
        'max_reservation': 0,
        'default_duration': float('inf'),
        'backfilling': 'ON',
        'prioritize_preemptees': False,
        'preemptees_keep_resources': False,
        'max_preemptees': 0,
        'preemption_distance': '00:15:00',
        'preemption_priority_adjustments': None,
    }

    BOOL_KEY_MAP = QconfObject.get_bool_key_map(REQUIRED_DATA_DEFAULTS)
    INT_KEY_MAP = QconfObject.get_int_key_map(REQUIRED_DATA_DEFAULTS)
    FLOAT_KEY_MAP = QconfObject.get_float_key_map(REQUIRED_DATA_DEFAULTS)
    DEFAULT_LIST_DELIMITER = ','
    LIST_KEY_MAP = {
        'job_load_adjustments': ',',
        'fair_urgency_list': ',',
    }
    DEFAULT_DICT_DELIMITER = ','
    DICT_KEY_MAP = {
        'usage_weight_list': ',',
        'halflife_decay_list': ':',
    }

    UGE_CASE_SENSITIVE_KEYS = {
        'schedd_job_info': str.lower,
    }

    def __init__(self, data=None, metadata=None, json_string=None):
        """ 
        Class constructor. 

        :param data: Configuration data. If provided, it will override corresponding data from JSON string representation.
        :type data: dict

        :param metadata: Configuration metadata. If provided, it will override corresponding metadata from JSON string representation.
        :type metadata: dict

        :param json_string: Configuration JSON string representation.
        :type json_string: str

        :raises: **InvalidArgument** - in case metadata is not a dictionary, JSON string is not valid, or it does not contain dictionary representing a SchedulerConfiguration object.
        """

        QconfObject.__init__(self, data=data, metadata=metadata, json_string=json_string)



#!/usr/bin/env python
#
# ___INFO__MARK_BEGIN__
#######################################################################################
# Copyright 2016-2024 Altair Engineering Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#######################################################################################
# ___INFO__MARK_END__
#

import tempfile

from .utils import needs_uge
from .utils import generate_random_string
from .utils import create_config_file
from .utils import load_values

from uge.api.qconf_api import QconfApi
from uge.config.config_manager import ConfigManager
from uge.log.log_manager import LogManager
from uge.exceptions.object_not_found import ObjectNotFound
from uge.exceptions.object_already_exists import ObjectAlreadyExists

create_config_file()
API = QconfApi()
CALENDAR_NAME = '%s' % generate_random_string(6)
CONFIG_MANAGER = ConfigManager.get_instance()
LOG_MANAGER = LogManager.get_instance()
VALUES_DICT = load_values('test_values.json')
print(VALUES_DICT)


@needs_uge
def test_object_not_found():
    try:
        calendar = API.get_cal('__non_existent_calendar__')
        assert (False)
    except ObjectNotFound as ex:
        # ok
        pass


def test_generate_cal():
    calendar = API.generate_cal(CALENDAR_NAME)
    assert (calendar.data['calendar_name'] == CALENDAR_NAME)


def test_add_cal():
    try:
        calendar_list = API.list_cals()
    except ObjectNotFound as ex:
        # no calendars defined
        calendar_list = []
    calendar = API.add_cal(name=CALENDAR_NAME)
    assert (calendar.data['calendar_name'] == CALENDAR_NAME)
    calendar_list2 = API.list_cals()
    assert (len(calendar_list2) == len(calendar_list) + 1)
    assert (calendar_list2.count(CALENDAR_NAME) == 1)


def test_list_cals():
    calendar_list = API.list_cals()
    assert (calendar_list is not None)


def test_object_already_exists():
    try:
        calendar = API.add_cal(name=CALENDAR_NAME)
        assert (False)
    except ObjectAlreadyExists as ex:
        # ok
        pass


def test_get_cal():
    calendar = API.get_cal(CALENDAR_NAME)
    assert (calendar.data['calendar_name'] == CALENDAR_NAME)


def test_generate_cal_from_json():
    cal = API.get_cal(CALENDAR_NAME)
    assert (cal.data['calendar_name'] == CALENDAR_NAME)
    json = cal.to_json()
    cal2 = API.generate_object(json)
    assert (cal2.__class__.__name__ == cal.__class__.__name__)
    for key in list(cal.data.keys()):
        v = cal.data[key]
        v2 = cal2.data[key]
        assert (str(v) == str(v2))


def test_modify_cal():
    calendar = API.get_cal(CALENDAR_NAME)
    week = calendar.data['week']
    calendar = API.modify_cal(name=CALENDAR_NAME, data={'week': '1-2'})
    week2 = calendar.data['week']
    assert (week2 == '1-2')


def test_get_cals():
    call = API.list_cals()
    cals = API.get_cals()
    for cal in cals:
        print("#############################################")
        print(cal.to_uge())
        assert (cal.data['calendar_name'] in call)


def test_write_cals():
    try:
        tdir = tempfile.mkdtemp()
        print("*************************** " + tdir)
        cal_names = VALUES_DICT['cal_names']
        cals = API.get_cals()
        for cal in cals:
            print("Before #############################################")
            print(cal.to_uge())

        new_cals = []
        for name in cal_names:
            ncal = API.generate_cal(name=name)
            new_cals.append(ncal)
        API.mk_cals_dir(tdir)
        API.write_cals(new_cals, tdir)
        API.add_cals_from_dir(tdir)
        API.modify_cals_from_dir(tdir)
        cals = API.get_cals()
        for cal in cals:
            print("After #############################################")
            print(cal.to_uge())

        cals = API.list_cals()
        for name in cal_names:
            assert (name in cals)
            print("calender found: " + name)

    finally:
        API.delete_cals_from_dir(tdir)
        API.rm_cals_dir(tdir)


def test_add_cals():
    try:
        new_cals = []
        cal_names = VALUES_DICT['cal_names']
        for name in cal_names:
            ncal = API.generate_cal(name=name)
            new_cals.append(ncal)

        # print all calendars currently in the cluster
        cals = API.get_cals()
        for cal in cals:
            print("Before #############################################")
            print(cal.to_uge())

        # add calendars
        API.add_cals(new_cals)
        API.modify_cals(new_cals)

        # print all calendars currently in the cluster
        cals = API.get_cals()
        for cal in cals:
            print("After #############################################")
            print(cal.to_uge())

        # check that calendars have been added
        cals = API.list_cals()
        for name in cal_names:
            assert (name in cals)
            print("calender found: " + name)

    finally:
        API.delete_cals(new_cals)

def test_delete_cal():
    calendar_list = API.list_cals()
    API.delete_cal(CALENDAR_NAME)
    try:
        calendar_list2 = API.list_cals()
    except ObjectNotFound as ex:
        # no calendars defined
        calendar_list2 = []
    assert (len(calendar_list2) == len(calendar_list) - 1)
    assert (calendar_list2.count(CALENDAR_NAME) == 0)



#!/usr/bin/env python
#
# ___INFO__MARK_BEGIN__
#######################################################################################
# Copyright 2016-2024 Altair Engineering Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#######################################################################################
# ___INFO__MARK_END__
#
from nose import SkipTest

from .utils import needs_uge
from .utils import generate_random_string
from .utils import create_config_file

from uge.api.ar_api import AdvanceReservationApi
from uge.api.qconf_api import QconfApi
from uge.config.config_manager import ConfigManager
from uge.log.log_manager import LogManager
from uge.exceptions.object_not_found import ObjectNotFound
from uge.exceptions.object_already_exists import ObjectAlreadyExists

create_config_file()
API = QconfApi()
PE_NAME = '%s.q' % generate_random_string(6)
QUEUE_NAME = '%s.q' % generate_random_string(6)
AR_API = AdvanceReservationApi()
# AR_NAME = '%s' % generate_random_string(6)
# CONFIG_MANAGER = ConfigManager.get_instance()
LOG_MANAGER = LogManager.get_instance()
logger = LOG_MANAGER.get_logger('test_ar')


@needs_uge
def test_request_ar():
    ar_id = AR_API.request_ar('-d 3600 -fr y')
    ar = AR_API.get_ar(ar_id)
    logger.info('AR: ', ar)
    assert (ar_id is not None)


def test_request_sr():
    ar_id = AR_API.request_ar('-cal_week mon-fri=8-16=on')
    ar = AR_API.get_ar(ar_id)
    logger.info('SR: ', ar)
    assert (ar_id is not None)


def test_get_ar_summary():
    ar_summary = AR_API.get_ar_summary()
    logger.info(ar_summary)
    assert (ar_summary is not None)


def test_get_ar():
    for ar_id in AR_API.get_ar_list():
        ar = AR_API.get_ar(str(ar_id))
        assert (ar is not None)
    else:
        raise SkipTest('There are no UGE advance reservations.')


def test_delete_all_ar():
    for ar_id in AR_API.get_ar_list():
        AR_API.delete_ar(ar_id)


def test_ar_per_pe_request():
    hl = API.list_shosts()
    assert (hl is not None)
    h = hl[0]
    assert (h is not None)

    slots = str(99999)

    pe = API.add_pe(name=PE_NAME, data={'slots': slots})
    assert (pe.data['pe_name'] == PE_NAME)

    q = API.add_queue(name=QUEUE_NAME, data={'slots': slots, 'pe_list': PE_NAME, 'hostlist': '@allhosts'})
    assert (q.data['qname'] == QUEUE_NAME)

    request = "-d 3600 -pe {} 4 -fr y -petask 0 -l h={}".format(PE_NAME, h)
    ar_id = AR_API.request_ar(request)
    ar = AR_API.get_ar(ar_id)
    logger.info('AR: ', ar)
    assert (ar is not None)
    pe_range = ar['qrstat']['ar_summary'][0]['resource_descriptor_list'][0]['pe_range']
    assert(pe_range == '0')

    # Cleanup
    AR_API.delete_ar(ar_id)
    API.delete_queue(QUEUE_NAME)
    API.delete_pe(PE_NAME)



#!/usr/bin/env python
# 
# ___INFO__MARK_BEGIN__ 
#######################################################################################
# Copyright 2016-2024 Altair Engineering Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#######################################################################################
# ___INFO__MARK_END__ 
# 
from nose import SkipTest

from .utils import needs_uge
from .utils import create_config_file

from uge.api.qconf_api import QconfApi
from uge.config.config_manager import ConfigManager
from uge.log.log_manager import LogManager
from uge.exceptions.object_not_found import ObjectNotFound
from uge.exceptions.object_already_exists import ObjectAlreadyExists
from uge.exceptions.authorization_error import AuthorizationError

create_config_file()
API = QconfApi()
CONFIG_MANAGER = ConfigManager.get_instance()
LOG_MANAGER = LogManager.get_instance()

@needs_uge
def test_list_ahosts():
    try:
        hl = API.list_ahosts()
        assert(hl is not None)
    except ObjectNotFound as ex:
        raise SkipTest('There are no configured UGE admin hosts.')


def test_object_already_exists():
    try:
        try:
            hl = API.list_ahosts()
            API.add_ahosts([hl[0]])
            assert(False)
        except ObjectNotFound as ex:
            raise SkipTest('There are no configured UGE admin hosts.')
    except ObjectAlreadyExists as ex:
        # ok
        pass


def test_delete_and_add_ahosts():
    try:
        hl = API.list_ahosts()
        host_name = None
        for h in hl:
            if h != CONFIG_MANAGER['host']:
                host_name = h
                break
        if not host_name:
            raise SkipTest('Could not find UGE admin host that could be removed.')
        try:
            hl2 = API.delete_ahosts([host_name])
            assert(hl2.data.count(host_name) == 0)
        except AuthorizationError as ex:
            raise SkipTest('Skip master host warning %s.' % str(ex))
        hl3 = API.add_ahosts(host_name)
        assert(hl3.data.count(host_name) == 1)
    except ObjectNotFound as ex:
        raise SkipTest('There are no configured UGE admin hosts.')



#!/usr/bin/env python
# 
# ___INFO__MARK_BEGIN__
#######################################################################################
# Copyright 2016-2024 Altair Engineering Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#######################################################################################
# ___INFO__MARK_END__
# 
import types
from .utils import needs_uge
from .utils import generate_random_string
from .utils import create_config_file

from uge.api.qconf_api import QconfApi
from uge.config.config_manager import ConfigManager
from uge.log.log_manager import LogManager
from uge.exceptions.object_not_found import ObjectNotFound
from uge.exceptions.object_already_exists import ObjectAlreadyExists

create_config_file()
API = QconfApi()
JC_NAME = 'jc%s' % generate_random_string(6)
CONFIG_MANAGER = ConfigManager.get_instance()
LOG_MANAGER = LogManager.get_instance()


@needs_uge
def test_object_not_found():
    try:
        jc = API.get_jc('__non_existent_jc__')
        assert (False)
    except ObjectNotFound as ex:
        # ok
        pass


def test_generate_jc():
    jc = API.generate_jc(JC_NAME)
    assert (jc.data['jcname'] == JC_NAME)


def test_add_jc():
    jc_list = API.list_jcs()
    jc = API.add_jc(name=JC_NAME)
    assert (jc.data['jcname'] == JC_NAME)
    jc_list2 = API.list_jcs()
    assert (len(jc_list2) == len(jc_list) + 1)
    assert (jc_list2.count(JC_NAME) == 1)


def test_list_jcs():
    jc_list = API.list_jcs()
    assert (jc_list is not None)


def test_object_already_exists():
    try:
        jc = API.add_jc(name=JC_NAME)
        assert (False)
    except ObjectAlreadyExists as ex:
        # ok
        pass


def test_get_jc():
    jc = API.get_jc(JC_NAME)
    assert (jc.data['jcname'] == JC_NAME)


def test_generate_jc_from_json():
    jc = API.get_jc(JC_NAME)
    json = jc.to_json()
    jc2 = API.generate_object(json)
    assert (jc2.__class__.__name__ == jc.__class__.__name__)
    for key in list(jc.data.keys()):
        v = jc.data[key]
        v2 = jc2.data[key]
        if type(v) == list:
            assert (len(v) == len(v2))
            for s in v:
                assert (v2.count(s) == 1)
        elif type(v) == dict:
            for key in list(v.keys()):
                assert (str(v[key]) == str(v2[key]))
        else:
            assert (str(v) == str(v2))


def test_modify_jc():
    jc = API.get_jc(JC_NAME)
    jc = API.modify_jc(name=JC_NAME, data={'CMDNAME': '/bin/ls'})
    shell = jc.data['CMDNAME']
    assert (shell == '/bin/ls')

def test_modify_petask_1():
    jc = API.get_jc(JC_NAME)
    jc = API.modify_jc(name=JC_NAME, data={'q_hard': '{+}UNSPECIFIED;0|{+~}{~}master.q'})
    q_hard = jc.data['q_hard']
    assert (q_hard == '{+}UNSPECIFIED;0|{+~}{~}master.q')

def test_modify_petask_2():
    jc = API.get_jc(JC_NAME)
    jc = API.modify_jc(name=JC_NAME, data={'l_hard': '{+~}{~}Z=true,[{+~}large={~}Y=true];0|{+~}{~}A=true;1,2|{+~}{~}B=true;3,4|{+}UNSPECIFIED,[{+~}large={~}B=true]'})
    l_hard = jc.data['l_hard']
    assert (l_hard == '{+~}{~}Z=true,[{+~}large={~}Y=true];0|{+~}{~}A=true;1,2|{+~}{~}B=true;3,4|{+}UNSPECIFIED,[{+~}large={~}B=true]')

def test_modify_petask_3():
    jc = API.get_jc(JC_NAME)
    jc = API.modify_jc(name=JC_NAME, data={'l_hard': '{+~}{~}Z=true,[{+~}large={~}Y=true];0|{+~}{~}A=true;1,2|{+~}{~}B=true;3,4|{+}UNSPECIFIED,[{+~}large={~}B=true]'})
    l_hard = jc.data['l_hard']
    assert (l_hard == '{+~}{~}Z=true,[{+~}large={~}Y=true];0|{+~}{~}A=true;1,2|{+~}{~}B=true;3,4|{+}UNSPECIFIED,[{+~}large={~}B=true]')


def test_delete_jc():
    jc_list = API.list_jcs()
    API.delete_jc(JC_NAME)
    jc_list2 = API.list_jcs()
    assert (len(jc_list2) == len(jc_list) - 1)
    assert (jc_list2.count(JC_NAME) == 0)



#!/usr/bin/env python
#
# ___INFO__MARK_BEGIN__
#######################################################################################
# Copyright 2016-2024 Altair Engineering Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#######################################################################################
# ___INFO__MARK_END__
#

import tempfile

from .utils import needs_uge
from .utils import generate_random_string
from .utils import create_config_file
from .utils import load_values

from uge.api.qconf_api import QconfApi
from uge.config.config_manager import ConfigManager
from uge.log.log_manager import LogManager
from uge.exceptions.object_not_found import ObjectNotFound
from uge.exceptions.object_already_exists import ObjectAlreadyExists

create_config_file()
API = QconfApi()
CKPT_NAME = '%s.q' % generate_random_string(6)
CONFIG_MANAGER = ConfigManager.get_instance()
LOG_MANAGER = LogManager.get_instance()
VALUES_DICT = load_values('test_values.json')
print(VALUES_DICT)


@needs_uge
def test_object_not_found():
    try:
        pe = API.get_ckpt('__non_existent_ckpt__')
        assert (False)
    except ObjectNotFound as ex:
        # ok
        pass


def test_generate_ckpt():
    ckpt = API.generate_ckpt(CKPT_NAME)
    assert (ckpt.data['ckpt_name'] == CKPT_NAME)


def test_add_ckpt():
    try:
        ckptl = API.list_ckpts()
    except ObjectNotFound as ex:
        # no ckpts defined
        ckptl = []
    ckpt = API.add_ckpt(name=CKPT_NAME)
    assert (ckpt.data['ckpt_name'] == CKPT_NAME)
    ckptl2 = API.list_ckpts()
    assert (len(ckptl2) == len(ckptl) + 1)
    assert (ckptl2.count(CKPT_NAME) == 1)


def test_list_ckpts():
    ckptl = API.list_ckpts()
    assert (ckptl is not None)


def test_object_already_exists():
    try:
        ckpt = API.add_ckpt(name=CKPT_NAME)
        assert (False)
    except ObjectAlreadyExists as ex:
        # ok
        pass


def test_get_ckpt():
    ckpt = API.get_ckpt(CKPT_NAME)
    assert (ckpt.data['ckpt_name'] == CKPT_NAME)


def test_generate_ckpt_from_json():
    ckpt = API.get_ckpt(CKPT_NAME)
    assert (ckpt.data['ckpt_name'] == CKPT_NAME)
    json = ckpt.to_json()
    ckpt2 = API.generate_object(json)
    assert (ckpt2.__class__.__name__ == ckpt.__class__.__name__)
    for key in list(ckpt.data.keys()):
        v = ckpt.data[key]
        v2 = ckpt2.data[key]
        assert (str(v) == str(v2))


def test_modify_ckpt():
    ckpt = API.get_ckpt(CKPT_NAME)
    ckpt_dir = ckpt.data['ckpt_dir']
    ckpt = API.modify_ckpt(name=CKPT_NAME, data={'ckpt_dir': '/storage'})
    ckpt_dir2 = ckpt.data['ckpt_dir']
    assert (ckpt_dir2 == '/storage')


def test_get_ckpts():
    ckptl = API.list_ckpts()
    ckpts = API.get_ckpts()
    for ckpt in ckpts:
        print("#############################################")
        print(ckpt.to_uge())
        assert (ckpt.data['ckpt_name'] in ckptl)


def test_write_ckpts():
    try:
        tdir = tempfile.mkdtemp()
        print("*************************** " + tdir)
        ckpt_names = VALUES_DICT['ckpt_names']
        ckpts = API.get_ckpts()
        for ckpt in ckpts:
            print("Before #############################################")
            print(ckpt.to_uge())

        new_ckpts = []
        for name in ckpt_names:
            nckpt = API.generate_ckpt(name=name)
            new_ckpts.append(nckpt)
        API.mk_ckpts_dir(tdir)
        API.write_ckpts(new_ckpts, tdir)
        API.add_ckpts_from_dir(tdir)
        API.modify_ckpts_from_dir(tdir)
        ckpts = API.get_ckpts()
        for ckpt in ckpts:
            print("After #############################################")
            print(ckpt.to_uge())

        ckpts = API.list_ckpts()
        for name in ckpt_names:
            assert (name in ckpts)
            print("ckpt found: " + name)

    finally:
        API.delete_ckpts_from_dir(tdir)
        API.rm_ckpts_dir(tdir)


def test_add_ckpts():
    try:
        new_ckpts = []
        ckpt_names = VALUES_DICT['ckpt_names']
        for name in ckpt_names:
            nckpt = API.generate_ckpt(name=name)
            new_ckpts.append(nckpt)

        # print all checkpoints currently in the cluster
        ckpts = API.get_ckpts()
        for ckpt in ckpts:
            print("Before #############################################")
            print(ckpt.to_uge())

        # add checkpoints
        API.add_ckpts(new_ckpts)
        API.modify_ckpts(new_ckpts)

        # print all checkpoints currently in the cluster
        ckpts = API.get_ckpts()
        for ckpt in ckpts:
            print("After #############################################")
            print(ckpt.to_uge())

        # check that checkpoints have been added
        ckpts = API.list_ckpts()
        for name in ckpt_names:
            assert (name in ckpts)
            print("ckpt found: " + name)

    finally:
        API.delete_ckpts(new_ckpts)


def test_delete_ckpt():
    ckptl = API.list_ckpts()
    API.delete_ckpt(CKPT_NAME)
    try:
        ckptl2 = API.list_ckpts()
    except ObjectNotFound as ex:
        # no ckpts defined
        ckptl2 = []
    assert (len(ckptl2) == len(ckptl) - 1)
    assert (ckptl2.count(CKPT_NAME) == 0)



#!/usr/bin/env python
#
# ___INFO__MARK_BEGIN__
#######################################################################################
# Copyright 2016-2024 Altair Engineering Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#######################################################################################
# ___INFO__MARK_END__
#
import re
import sys
from uge.log.log_manager import LogManager
from uge.exceptions.object_not_found import ObjectNotFound
from uge.exceptions.invalid_request import InvalidRequest
from uge.exceptions.object_already_exists import ObjectAlreadyExists
from uge.exceptions.invalid_argument import InvalidArgument
from uge.objects.qconf_object import QconfObject
from uge.objects.qconf_name_list import QconfNameList


class ListBasedObjectManager(object):
    QCONF_ERROR_REGEX_LIST = [
        (re.compile('.*already exists.*'), ObjectAlreadyExists),
        (re.compile('.*does not exist.*'), ObjectNotFound),
        (re.compile('.*may not remove.*'), InvalidRequest),
    ]

    OBJECT_NAME = None
    OBJECT_CLASS_UGE_NAME = None

    def __init__(self, qconf_executor):
        self.logger = LogManager.get_instance().get_logger(self.__class__.__name__)
        self.qconf_executor = qconf_executor

    def __prepare_names(self, names):
        if sys.version_info < (3,):
            text_type = unicode
            binary_type = str
        else:
            text_type = str
            binary_type = bytes
        if type(names) == text_type or type(names) == binary_type:
            if names.find(','):
                name_list = names.split(',')
            else:
                name_list = names.split()
        elif type(names) == list:
            name_list = names
        else:
            raise InvalidArgument(
                'Names must be provided either as a list, or as a string containing names separated by space or comma.')
        name_list2 = []
        for name in name_list:
            trimmed_name = name.strip()
            if len(trimmed_name):
                name_list2.append(trimmed_name)
        return ','.join(name_list2)

    def add_names(self, names):
        names = self.__prepare_names(names)
        self.qconf_executor.execute_qconf('-a%s %s' % (self.OBJECT_CLASS_UGE_NAME, names), self.QCONF_ERROR_REGEX_LIST,
                                          combine_error_lines=False)
        name_list = self.list_names()
        name_list.set_modify_metadata()
        return name_list

    def delete_names(self, names):
        names = self.__prepare_names(names)
        self.qconf_executor.execute_qconf('-d%s %s' % (self.OBJECT_CLASS_UGE_NAME, names), self.QCONF_ERROR_REGEX_LIST,
                                          combine_error_lines=False)
        name_list = self.list_names()
        name_list.set_modify_metadata()
        return name_list

    def list_names(self):
        try:
            qconf_output = self.qconf_executor.execute_qconf('-s%s' % (self.OBJECT_CLASS_UGE_NAME),
                                                             self.QCONF_ERROR_REGEX_LIST).get_stdout()
            name_list = QconfNameList(metadata={'description': 'List of %s names' % (self.OBJECT_NAME)},
                                      data=QconfObject.get_list_from_qconf_output(qconf_output))
        except ObjectNotFound:
            name_list = QconfNameList(metadata={'description': 'List of %s names' % (self.OBJECT_NAME)}, data=[])
        return name_list


#############################################################################
# Testing.
if __name__ == '__main__':
    pass



#!/usr/bin/env python
#
# ___INFO__MARK_BEGIN__
#######################################################################################
# Copyright 2016-2024 Altair Engineering Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#######################################################################################
# ___INFO__MARK_END__
#
import os
import subprocess

from uge.log.log_manager import LogManager
from uge.exceptions.command_failed import CommandFailed


class UgeSubprocess(subprocess.Popen):

    def __init__(self, args, bufsize=0, executable=None, stdin=None, stdout=subprocess.PIPE, stderr=subprocess.PIPE,
                 preexec_fn=None, close_fds=False, shell=True, cwd=None, env=None, universal_newlines=False,
                 startupinfo=None, creationflags=0, use_exceptions=True):
        """
        Overrides Popen constructor with defaults more appropriate for
        Uge usage.
        """
        subprocess.Popen.__init__(self, args, bufsize, executable, stdin, stdout, stderr, preexec_fn, close_fds, shell,
                                  cwd, env, universal_newlines, startupinfo, creationflags)
        self.logger = LogManager.get_instance().get_logger(self.__class__.__name__)
        self.stdout_ = None
        self.stderr_ = None
        self.args_ = args
        self.use_exceptions = use_exceptions

    def __command_log(self):
        # Not very useful to show the name of this file.
        # Walk up the stack to find the caller.
        import traceback
        stack = traceback.extract_stack()
        for i in range(2, len(stack)):
            if stack[-i][0] != stack[-1][0]:
                fileName, lineNumber, functionName, text = stack[-i]
                break
        else:
            fileName = lineNumber = functionName = text = '?'

        self.logger.debug('from [%s:%s] Invoking: [%s]' %
                          (os.path.basename(fileName), lineNumber, self.args_))

    def run(self, input=None):
        self.__command_log()
        (self.stdout_, self.stderr_) = subprocess.Popen.communicate(self, input)
        self.logger.debug('Exit status: %s' % self.returncode)
        if self.returncode != 0 and self.use_exceptions:
            self.logger.debug('StdOut: %s' % self.stdout_.decode())
            self.logger.debug('StdErr: %s' % self.stderr_.decode())
            raise CommandFailed(self.stderr_.decode(), self.stdout_.decode(), self.stderr_.decode(), self.returncode)
        return self.stdout_.decode(), self.stderr_.decode()

    def get_logger(self):
        return self.logger

    def get_args(self):
        return self.args_

    def get_stdout(self):
        return self.stdout_.decode()

    def get_stderr(self):
        return self.stderr_.decode()

    def get_exit_status(self):
        return self.returncode

    @classmethod
    def execute(cls, command):
        """ Create subprocess and run it, return subprocess object. """
        p = UgeSubprocess(command)
        p.run()
        return p

    @classmethod
    def execute_and_ignore_failure(cls, command):
        """ 
        Create subprocess, run it, ignore failures, return subprocess object. 
        """
        p = UgeSubprocess(command)
        try:
            p.run()
        except CommandFailed as ex:
            p.get_logger().debug('Command failed, stdout: %s, stderr: %s' % (p.get_stdout(), p.get_stderr()))
        return p

    @classmethod
    def execute_and_log_to_stdout(cls, command):
        """
        Executes command, displays output to stdout and maintains log file.
        Returns subprocess object
        """
        p = UgeSubprocess(command)
        while True:
            outp = p.stdout.readline()
            if not outp:
                break
            # print(outp, end=' ')
            print(outp)
        retval = p.wait()

        p.logger.debug('Exit status: %s' % retval)

        if retval != 0:
            emsg = ''
            while True:
                err = p.stderr.readline()
                if not err:
                    break
                emsg += err
            raise CommandFailed(emsg)

        return p


#############################################################################
# Testing.
if __name__ == '__main__':
    p = UgeSubprocess('ls -l', use_exceptions=False)
    # print(p.run())
    p.run()
    print(p.get_stdout())
    print(p.get_stderr())
    print(p.get_exit_status())



#!/usr/bin/env python
#
# ___INFO__MARK_BEGIN__
#######################################################################################
# Copyright 2016-2024 Altair Engineering Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#######################################################################################
# ___INFO__MARK_END__
#

from uge.exceptions.qconf_exception import QconfException
from uge.constants import uge_status


class CommandFailed(QconfException):
    """ 
    Command failed error class.

    Error code: uge_status.UGE_COMMAND_FAILED
    """

    def __init__(self, error='', command_stdout=None, command_stderr=None, command_exit_status=None, **kwargs):
        """ 
        Class constructor. 

        :param error: Error message.
        :type error: str

        :param command_stdout: Command standard output.
        :type command_stdout: str

        :param command_stderr: Command standard error.
        :type command_stderr: str

        :param command_exit_status: Command exit status.
        :type command_exit_status: int

        :param kwargs: Keyword arguments, may contain 'args=error_message', 'exception=exception_object', or 'error_details=details'.
        """
        QconfException.__init__(
            self, error, uge_status.UGE_COMMAND_FAILED,
            **kwargs)
        self.command_stdout = command_stdout
        self.command_stderr = command_stderr
        self.command_exit_status = command_exit_status

    def get_command_stdout(self):
        """
        :returns: Command standard output.
        """
        return self.command_stdout

    def get_command_stderr(self):
        """
        :returns: Command standard error.
        """
        return self.command_stderr

    def get_command_exit_status(self):
        """
        :returns: Command exit status.
        """
        return self.command_exit_status



#!/usr/bin/env python
#
# ___INFO__MARK_BEGIN__
#######################################################################################
# Copyright 2016-2024 Altair Engineering Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#######################################################################################
# ___INFO__MARK_END__
#

__docformat__ = 'reStructuredText'

import os
import collections
try:
    collectionsAbc = collections.abc
except AttributeError:
    collectionsAbc = collections
from functools import wraps
from decorator import decorator
from uge.log.log_manager import LogManager
from uge.exceptions.ar_exception import AdvanceReservationException
from uge.exceptions.configuration_error import ConfigurationError
from uge.api.impl.qrstat_executor import QrstatExecutor
from uge.api.impl.qrsub_executor import QrsubExecutor
from uge.api.impl.qrdel_executor import QrdelExecutor
from uge.api.impl.ar_manager import AdvanceReservationManager
from uge.objects.ar_object_factory import AdvanceReservationObjectFactory


class AdvanceReservationApi(object):
    """ High-level advance reservation API class. """

    DEFAULT_SGE_CELL = 'default'
    DEFAULT_SGE_QMASTER_PORT = 6444
    DEFAULT_SGE_EXECD_PORT = 6445

    logger = None

    def __init__(self, sge_root=None, sge_cell=None,
                 sge_qmaster_port=None, sge_execd_port=None):
        """ 
        Class constructor. 

        :param sge_root: SGE root directory. It can be set via environment variable SGE_ROOT. 
        :type sge_root: str

        :param sge_cell: SGE cell name. It can be set via environment variable SGE_CELL. Default cell name is 'default'.
        :type sge_cell: str

        :param sge_qmaster_port: SGE Qmaster port. It can be set via environment variable SGE_QMASTER_PORT. Default port is 6444. 
        :type sge_qmaster_port: int

        :param sge_execd_port: SGE Execd port. It can be set via environment variable SGE_EXECD_PORT. Default port is 6445.
        :type sge_execd_port: int

        :raises ConfigurationError: in case sge_root is not provided, and environment variable SGE_ROOT is not defined.
        :raises QmasterUnreachable: in case UGE Qmaster cannot be reached.
        :raises AdvanceReservationException: for any other errors.

        >>> api = AdvanceReservationApi(sge_root='/opt/uge')
        """
        self.__configure(sge_root, sge_cell, sge_qmaster_port, sge_execd_port)

    def __configure(self, sge_root, sge_cell, sge_qmaster_port,
                    sge_execd_port):
        self.get_logger()
        if not sge_root:
            sge_root = os.environ.get('SGE_ROOT')
            if not sge_root:
                raise ConfigurationError('SGE_ROOT is not defined.')
        if not sge_cell:
            sge_cell = os.environ.get('SGE_CELL', self.DEFAULT_SGE_CELL)
        if not sge_qmaster_port:
            sge_qmaster_port = os.environ.get('SGE_QMASTER_PORT', self.DEFAULT_SGE_QMASTER_PORT)
        if not sge_execd_port:
            sge_execd_port = os.environ.get('SGE_EXECD_PORT', self.DEFAULT_SGE_EXECD_PORT)

        self.logger.debug('Configuration: SGE_ROOT=%s, SGE_CELL=%s, SGE_QMASTER_PORT=%s, SGE_EXECD_PORT=%s' % (
        sge_root, sge_cell, sge_qmaster_port, sge_execd_port))
        self.qrstat_executor = QrstatExecutor(
            sge_root=sge_root, sge_cell=sge_cell,
            sge_qmaster_port=sge_qmaster_port,
            sge_execd_port=sge_qmaster_port)
        self.qrsub_executor = QrsubExecutor(
            sge_root=sge_root, sge_cell=sge_cell,
            sge_qmaster_port=sge_qmaster_port,
            sge_execd_port=sge_qmaster_port)
        self.qrdel_executor = QrdelExecutor(
            sge_root=sge_root, sge_cell=sge_cell,
            sge_qmaster_port=sge_qmaster_port,
            sge_execd_port=sge_qmaster_port)
        self.ar_manager = AdvanceReservationManager(self.qrstat_executor, self.qrsub_executor, self.qrdel_executor)

    @classmethod
    def get_logger(cls):
        if not cls.logger:
            cls.logger = LogManager.get_instance().get_logger(cls.__name__)
        return cls.logger

    def api_call2(func):
        @wraps(func)
        def wrapped_call(*args, **kwargs):
            try:
                result = func(*args, **kwargs)
                return result
            except AdvanceReservationException:
                raise
            except Exception as ex:
                raise AdvanceReservationException(exception=ex)

        return decorator(wrapped_call, func)

    # Make sure only ar specific exceptions are raised
    # Use two decorators to keep signature for documentation
    def api_call(*dargs, **dkwargs):
        def internal_call(func):
            @wraps(func)
            def wrapped_call(func, *args, **kwargs):
                try:
                    result = func(*args, **kwargs)
                    return result
                except AdvanceReservationException as ex:
                    raise
                except Exception as ex:
                    raise AdvanceReservationException(exception=ex)

            return decorator(wrapped_call, func)

        if len(dargs) == 1 and isinstance(dargs[0], collectionsAbc.Callable):
            return internal_call(dargs[0])
        return internal_call

    def get_uge_version(self):
        """ Get version of UGE qmaster that API is connected to.

        :returns: UGE version string.

        :raises AdvanceReservationException: in case of any errors.
        """
        return self.qrstat_executor.get_uge_version()

    def generate_object(self, json_string, target_uge_version=None):
        """ Use specified JSON string to generate object for the target UGE version.
 
        :param json_string: input object's JSON string representation 
        :type json_string: str

        :param target_uge_version: target UGE version
        :type json_string: str

        :returns: Generated Qconf object.

        :raises AdvanceReservationException: in case of any errors.
        """
        return AdvanceReservationObjectFactory.generate_object(json_string, target_uge_version)

    @api_call
    def get_ar(self, name):
        """ Retrieve UGE advance reservation info.

        :param name: ar id.
        :type name: str

        :returns: ar as json string

        :raises ObjectNotFound: in case cluster queue object with a given name does not exist.
        :raises QmasterUnreachable: in case UGE Qmaster cannot be reached.
        :raises AdvanceReservationException: for any other errors.

        >>> ar = api.get_ar('123')
        >>> all_q.data['load_thresholds']
        'np_load_avg=2.0'
        """
        return self.qrstat_executor.get_ar(name)

    @api_call
    def get_ar_summary(self):
        """ List UGE advance reservations.

        :returns: advance reservation summary.

        :raises QmasterUnreachable: in case UGE Qmaster cannot be reached.
        :raises AdvanceReservationException: for any other errors.

        >>> ar_summary = api.get_ar_summary()
        >>> ar_summary
        ['all.q']
        >>> ar_summary.__class__.__name__
        'QconfNameList'
        """
        return self.qrstat_executor.get_ar_summary()

    @api_call
    def get_ar_list(self):
        """ List UGE advance reservation ids.
        :returns: advance reservation id list.

        :raises QmasterUnreachable: in case UGE Qmaster cannot be reached.
        :raises AdvanceReservationException: for any other errors.

        >>> ar_summary = api.get_ar_list()
        >>> ar_summary
        ['all.q']
        >>> ar_summary.__class__.__name__
        'QconfNameList'
        """
        return self.qrstat_executor.get_ar_list()

    @api_call
    def request_ar(self, argstr):
        """ Request UGE advance reservation.

        :param pycl_object: Cluster queue object to be added.
        :type pycl_object: ClusterQueue 

        :param name: Cluster queue name; if provided, it will override queue name in the provided cluster queue object, data dictionary, or in the json string.
        :type name: str

        :param data: Queue data dictionary; if provided, dictionary values will override corresponding values from the cluster queue object, or from the JSON string.
        :type data: dict

        :param metadata: Queue metadata dictionary.
        :type metadata: dict

        :param json_string: JSON string representation of the queue object to be added.
        :type json_string: str

        :returns: Newly added advance reservation id object.

        :raises InvalidArgument: if provided PYCL object is not an instance of ClusterQueue, or data is not a dictionary, or if json_string does not represent a valid JSON object.
        :raises InvalidRequest: if provided arguments do not specify cluster queue name.
        :raises ObjectAlreadyExists: in case cluster queue object with a given name already exists.
        :raises ObjectNotFound: in case an object contained in the cluster queue data does not exist.
        :raises AuthorizationError: if user is not authorized to make changes to UGE configuration.
        :raises QmasterUnreachable: in case UGE Qmaster cannot be reached.
        :raises AdvanceReservationException: for any other errors.

        >>> new_q = api.request_ar(name='new.q', data={'slots' : 3})
        >>> 123
        <uge.objects.cluster_queue_v1_0.ClusterQueue object at 0x7fc32e14ef50>
        >>> new_q.data['slots']
        3
        """
        return self.qrsub_executor.request_ar(argstr)

    @api_call
    def delete_ar(self, name):
        """ Delete UGE advance reservation.

        :param name: ar id.
        :type name: str

        :raises ObjectNotFound: in case advance reservation object with a given name does not exist.
        :raises QmasterUnreachable: in case UGE Qmaster cannot be reached.
        :raises AdvanceReservationException: for any other errors.

        >>> api.delete_ar('123')
        """
        return self.qrdel_executor.delete_ar(name)


#############################################################################
# Testing.
if __name__ == '__main__':
    lm = LogManager.get_instance()
    lm.set_console_log_level('trace')
    api = AdvanceReservationApi()
    print(api.get_uge_version())
    ar_id1 = api.request_ar('-d 3600 -fr y')
    ar1 = api.get_ar(ar_id1)
    print('AR 1: ', ar1)
    ar_id2 = api.request_ar('-cal_week mon-fri=8-16=on')
    ar2 = api.get_ar(ar_id2)
    print('AR 2: ', ar2)
    ar_summary = api.get_ar_summary()
    print('AR SUMMARY: ', ar_summary)
    print('AR LIST: ', api.get_ar_list())
    print(api.delete_ar(ar_id1))
    print(api.delete_ar(ar_id2))
